--
-- SysMT cego table definitions
--

-- business group
drop if exists table sysmt_bgroup;
create table sysmt_bgroup(bgid int not null,
       bgname string(100) not null, 
       bgdesc string(10000));

create primary index on sysmt_bgroup(bgid);
create unique index sysmt_bgroup_i1 on sysmt_bgroup(bgname);

-- business group right
drop if exists table sysmt_bgright;
create table sysmt_bgright(bgid int not null,
       bgright string(20) not null);

create primary index on sysmt_bgright(bgid, bgright);

-- service catalog
drop if exists table sysmt_catalog;
create table sysmt_catalog(
       catid int not null,
       catname string(100) not null,
       category string(20) not null,
       srvtype string(100) not null,
       catdesc string(1000),
       catstatus string(20) not null);

create primary index on sysmt_catalog(catid);

-- custom search condition
drop if exists table sysmt_custcond;
create table sysmt_custcond(ccid int not null,
       ccname string(20) not null,
       ccspec string(100),
       usrid int not null,
       cctype string(20));

create primary index on sysmt_custcond(ccid);

-- db system
drop if exists table sysmt_dbsystem;
create table sysmt_dbsystem(sysid int not null, 
       dbtype string(10),
       dbversion string(20));
create primary index on sysmt_dbsystem(sysid);

-- db instance
drop if exists table sysmt_dbinstance;
create table sysmt_dbinstance(instid int not null,
       dbhost string(50));

-- devices
drop if exists table sysmt_device;
create table sysmt_device(
       devid int not null,
       devname string(100) not null,
       devserial string(100),
       devstatus string(20),
       devtype string(20),
       devdesc string(1000),
       rckid int);
       
create primary index on sysmt_device(devid);

drop if exists table sysmt_devicecomp;
create table sysmt_devicecomp(
       cpid int not null,
       devid int not null,
       coid int not null,
       compnum int not null);

create primary index on sysmt_devicecomp(cpid);

drop if exists table sysmt_comptype;
create table sysmt_comptype(
       coid int not null,
       compclass string(20) not null,
       comptype string(20) not null);

create primary index on sysmt_comptype(coid);

drop if exists table sysmt_dev2srv;
create table sysmt_dev2srv(
       devid int not null,	
       srvtype string(100) not null);

create primary index on sysmt_dev2srv(devid, srvtype);

-- datacenter infrastructure
drop if exists table sysmt_rack;
create table sysmt_rack(
       rckid int not null,
       rackname string(100) not null,
       locid int);
create primary index on sysmt_rack(rckid);

drop if exists table sysmt_location;
create table sysmt_location(
       locid int not null,
       locname string(100) not null,
       locdesc string(500));
create primary index on sysmt_location(locid);

-- document
drop if exists table sysmt_document;
create table sysmt_document(docid int not null,
       doctype string(100) not null,
       docname string(200) not null);

create primary index on sysmt_document(docid);

-- document 2 thread
drop if exists table sysmt_doc2thread;
create table sysmt_doc2thread(docid int not null, 
       thrid int not null);
create primary index on sysmt_doc2thread(docid, thrid);

-- file system template
drop if exists table sysmt_filesystmpl;
create table sysmt_filesystmpl(tmplid int not null, 
       fstmplname string(50),
       fstmpldesc string(500),
       fstmplrule string(200),
       fstmpltype string(10));

-- file system template entry
drop if exists table sysmt_filesystmplentry;
create table sysmt_filesystmplentry(tmplid int not null, 
       fstype string(50), 
       fspath string(100) not null, 
       fsvg string(100) not null, 
       fssizesmall long not null, 
       fsopt string(100), 
       fsid int, 
       fssizebig long, 
       fslv string(100));

-- firewall port set
drop if exists table sysmt_fwps;
create table sysmt_fwps(psid int not null,
       psname string(30),
       psdesc string(500));

create primary index on sysmt_fwps(psid);

-- firewall port set entry
drop if exists table sysmt_fwpsentry;
create table sysmt_fwpsentry(psid int, 
       portval string(30),
       porttype string(10),
       portdesc string(500));

-- firewall server group
drop if exists table sysmt_fwsg;
create table sysmt_fwsg(sgid int not null, 
       sgname string(30),
       sgdesc string(500));

create primary index on sysmt_fwsg(sgid);

-- firewall server group entry
drop if exists table sysmt_fwsgentry;
create table sysmt_fwsgentry(sgid int,
       ipval string(30),
       ipdesc string(500));

-- group search
drop if exists table sysmt_groupsearch;
create table sysmt_groupsearch(gsid int not null, 
       gsname string(20), 
       gsspec string(100));
create primary index on sysmt_groupsearch(gsid);

-- group search 2 group
drop if exists table sysmt_gs2bgrp;
create table sysmt_gs2bgrp(gsid int not null, 
       bgid int not null);
create primary index on sysmt_gs2bgrp(gsid, bgid);

-- identificator
drop if exists table sysmt_id;
create table sysmt_id(id string(10) not null, 
       idval int not null);
create primary index on sysmt_id(id);

-- system
drop if exists table sysmt_instance;
create table sysmt_instance(instid int not null, 
       instname string(50) not null, 
       sysid int not null);
create primary index on sysmt_instance(instid);

-- maintain
drop if exists table sysmt_mt;
create table sysmt_mt(mid int not null,
       shortdesc string(100),
       description string(500),
       mtfrom datetime,
       mtto datetime,
       status string(10) not null,
       ticketnr bigint(10),
       sysid int not null,
       lockby string(20),
       mttype string(10),
       downfrom datetime,
       downto datetime,
       thrid int);

create primary index on sysmt_mt(mid);
-- create index sysmt_mt_i1 on sysmt_mt(status);
create index sysmt_mt_i2 on sysmt_mt(mtfrom);
create index sysmt_mt_i3 on sysmt_mt(sysid);
create index sysmt_mt_i4 on sysmt_mt(mtto);

-- maintain approval
drop if exists table sysmt_mtapproval;
create table sysmt_mtapproval(mid int not null, bgid int not null, status string(10) not null);
create index sysmt_mtapproval_i1 on sysmt_mtapproval(mid);

-- maintain log
drop if exists table sysmt_mtlog;
create table sysmt_mtlog(logts datetime not null, mid int not null, logentry string(100));

-- maintain verification
drop if exists table sysmt_mtverification;
create table sysmt_mtverification(mid int not null,
       bgid int not null,
       status string(10) not null);

drop if exists table sysmt_orders;
create table sysmt_orders(
       ordid int not null,
       orddate datetime not null,
       orddesc string(100),
       ordby string(50));

drop if exists table sysmt_registeredsystem;
create table sysmt_registeredsystem(
       sysid int not null,
       registeredAt string(30) not null);

create primary index on sysmt_registeredsystem(sysid, registeredAt);

-- sap system
drop if exists table sysmt_sapsystem;
create table sysmt_sapsystem(sysid int not null, 
       sapsid string(50),
       saptype string(50),
       dbtype string(50), 
       dbversion string(50),
       sapversion string(50));
create primary index on sysmt_sapsystem(sysid);

-- sap instance
drop if exists table sysmt_sapinstance;
create table sysmt_sapinstance(instid int not null,
       instnr string(10),
       instsmd string(10),
       instsaptype string(10),
       instjavaport string(10),
       instdisp string(10));

create primary index on sysmt_sapinstance(instid);

-- sap addon
drop if exists table sysmt_sapaddon;
create table sysmt_sapaddon(aoid int not null, 
       aotype string(20), 
       aoname string(20),
       aohome string(100),
       aouser string(100),
       aogroup string(100),
       aoexepath string(100),
       aologpath string(100), 
       sysid int);
create primary index on sysmt_sapaddon(aoid);

-- sap displacement system
drop if exists table sysmt_sapdisp;
create table sysmt_sapdisp(sysid int not null, 
       dispsysid int not null);
create primary index on sysmt_sapdisp(sysid, dispsysid);

-- sap filesystem
drop if exists table sysmt_sapfilesys;
create table sysmt_sapfilesys(fstype string(50),
       fspath string(100) not null,
       fsvg string(100) not null,
       fssize long not null,
       sysid int not null,
       fsopt string(100),
       fslv string(100));

-- sap nfs
drop if exists table sysmt_sapnfs;
create table sysmt_sapnfs(nfsid int not null,
       nfspath string(100),
       nfsnode string(100),
       nfsmount string(100),
       sysid int);

-- service
drop if exists table sysmt_service;
create table sysmt_service(srvid int not null,
       srvname string(100) not null,
       srvtype string(100) not null,
       srvdesc string(1000),
       srvmail string(100),
       slaid int not null);

create primary index on sysmt_service(srvid);
create unique index sysmt_service_i1 on sysmt_service(srvname);

-- service 2 approval
drop if exists table sysmt_srv2approval;
create table sysmt_srv2approval(srvid int not null, 
       bgid int not null);
create primary index on sysmt_srv2approval(srvid, bgid);

-- service 2 group
drop if exists table sysmt_srv2bgrp;
create table sysmt_srv2bgrp(srvid int not null,
       bgid int not null);

create primary index on sysmt_srv2bgrp(srvid,bgid);
create index sysmt_srv2bgrp_i1 on sysmt_srv2bgrp(bgid);
create index sysmt_srv2bgrp_i2 on sysmt_srv2bgrp(srvid);


-- service 2 notification
drop if exists table sysmt_srv2notification;
create table sysmt_srv2notification(srvid int not null,
       bgid int not null);

create primary index on sysmt_srv2notification(srvid,bgid);

-- service 2 workflow
drop if exists table sysmt_srv2workflow;
create table sysmt_srv2workflow(srvid int not null,
       bgid int not null,
       seqnr int not null,
       wfaction string(100));

-- service 2 verification
drop if exists table sysmt_srv2verification;
create table sysmt_srv2verification(srvid int not null,
       bgid int not null);

-- SLA
drop if exists table sysmt_sla;
create table sysmt_sla(slaid int not null,
       slanr string(30) not null,
       shortdesc string(100));

create primary index on sysmt_sla(slaid);
create unique index sysmt_sla_i1 on sysmt_sla(slanr);


-- staff
drop if exists table sysmt_crew;
create table sysmt_crew(cmid int not null,
       	     membername string(50),
       	     description string(1000));

create primary index on sysmt_crew(cmid);

-- skill
drop if exists table sysmt_skill;
create table sysmt_skill(skid int not null,
       skname  string(30),
       skdesc string(1000));

create primary index on sysmt_skill(skid);

-- crew 2 srv
drop if exists table sysmt_crew2srv;
create table sysmt_crew2srv(cmid int not null,
       srvid int not null);

create primary index on sysmt_crew2srv(cmid, srvid);

-- crew 2 skill
drop if exists table sysmt_crew2skill;
create table sysmt_crew2skill(cmid int not null,
       skid int not null);

create primary index on sysmt_crew2skill(cmid, skid);

-- staff plan
drop if exists table sysmt_crewplan;
create table sysmt_crewplan(
       cpeid int not null,
       activity string(200),
       plannedfrom datetime,
       plannedto datetime,
       cmid int not null,
       mid int not null);

create primary index on sysmt_crewplan(cpeid);

-- system firewall entry
drop if exists table sysmt_sysfw;
create table sysmt_sysfw(fwrid int not null,
       sgidfrom int,
       sgidto int,
       psid int,
       sysid int,
       fwrdesc string(500));

create primary index on sysmt_sysfw(fwrid);

-- system
drop if exists table sysmt_system;
create table sysmt_system(sysid int not null, 
       sid string(50) not null, 
       sysrole string(20),
       sysdesc string(1000),
       srvid int not null,
       mtinfo string(300),
       status string(10) not null);
create primary index on sysmt_system(sysid);
create unique index sysmt_system_i1 on sysmt_system(sid);
create index sysmt_system_i2 on sysmt_system(srvid);


-- system 2 set
drop if exists table sysmt_sys2set;
create table sysmt_sys2set(sysid int not null,
       setid int not null);

create primary index on sysmt_sys2set(sysid, setid);

-- system set
drop if exists table sysmt_systemset;
create table sysmt_systemset(setid int not null, 
       setname string(100) not null,
       description string(1000));

create primary index on sysmt_systemset(setid);
create unique index sysmt_systemset_i1 on sysmt_systemset(setname);

-- system set 2 group
drop if exists table sysmt_set2bgrp;
create table sysmt_set2bgrp(setid int not null, bgid int not null);
create primary index on sysmt_set2bgrp(setid, bgid);

-- syslog
drop if exists table sysmt_syslog;
create table sysmt_syslog(sysid int not null, 
       ts datetime default sysdate not null,
       msg string(300));
create primary index on sysmt_syslog(sysid, ts);

-- system workflow
drop if exists table sysmt_sysworkflow;
create table sysmt_sysworkflow(sysid int not null,
       bgid int not null,
       seqnr int not null,
       wfaction string(100) not null,
       assigned datetime,
       released datetime,
       dbuser string(50),
       wfcomment string(500),
       status string(10) not null);

create primary index on sysmt_sysworkflow(sysid, bgid, seqnr);

-- system parameters
drop if exists table sysmt_sysparam;
create table sysmt_sysparam(pname string(20) not null,
       pval string(200));
create primary index on sysmt_sysparam(pname);


-- service modules
drop if exists table sysmt_catalogmodule;
create table sysmt_catalogmodule(
       camid int not null,
       catid int not null,
       coid int not null,
       moddesc string(100),
       modfee fixed(2) not null,
       modnum int not null);

create primary index on sysmt_catalogmodule(camid);
       
drop if exists table sysmt_sysresource;
create table sysmt_sysresource(
       resid int not null,
       instid int not null,
       ordid int not null,
       cpid int,
       resdesc string(100),	
       resnum int,
       monthlyfee fixed(2),
       claimedfrom datetime,
       claimedto datetime);

create primary index on sysmt_sysresource(resid);


-- thread group 2 business group
drop if exists table sysmt_tg2bgrp;
create table sysmt_tg2bgrp(tgid int not null, bgid int not null);

create primary index on sysmt_tg2bgrp(tgid,bgid);

-- thread
drop if exists table sysmt_thread;
create table sysmt_thread(thrid int not null,
       shortdesc string(100) not null,
       description string(1000),
       created datetime,
       remind datetime,
       lastedit datetime,
       lasteditby string(20), 
       status string(15) not null,
       tgid int,
       bgid int);

create primary index on sysmt_thread(thrid);
create index sysmt_thread_i1 on sysmt_thread(tgid);

-- thread rstat
drop if exists table sysmt_threadstat;
create table sysmt_threadstat(thrid int not null,
       usrid int not null,
       readstat string(15));

create primary index on sysmt_threadstat(thrid, usrid);

-- thread group
drop if exists table sysmt_threadgroup;
create table sysmt_threadgroup(tgid int not null,
       tgname string(30) not null,
       tgdesc string(500));
create primary index on sysmt_threadgroup(tgid);

-- thread log
drop if exists table sysmt_threadlog;
create table sysmt_threadlog(logts datetime,
       logentry string(1000),
       thrid int not null,
       dbuser string(50));

-- user
drop if exists table sysmt_user;
create table sysmt_user(usrid int not null,
       dbuser string(50) not null,
       password string(100),
       hostinfo string(100),
       username string(100),
       userdesc string(1000),
       userright string(10) not null,
       usermail string(100));

create primary index on sysmt_user(usrid);
create unique index sysmt_user_i1 on sysmt_user(dbuser);

-- user 2 group
drop if exists table sysmt_usr2bgrp;
create table sysmt_usr2bgrp(usrid int not null,
       bgid int not null);

create primary index on sysmt_usr2bgrp(usrid,bgid);
create index sysmt_usr2bgrp_i1 on sysmt_usr2bgrp(bgid);


-- user search
drop if exists table sysmt_usersearch;
create table sysmt_usersearch(usid int not null, usname string(50) not null, usspec string(1000), usrid int);


-- web system
drop if exists table sysmt_websystem;
create table sysmt_websystem(sysid int not null, 
       webtype string(10),
       webversion string(20));
create primary index on sysmt_websystem(sysid);

-- web instance
drop if exists table sysmt_webinstance;
create table sysmt_webinstance(instid int not null,
       webhost string(50));

-- scheduler
drop if exists table sysmt_schedule;
create table sysmt_schedule(schedid int not null,
       isactive string(5),
       schedentry string(200));

--
-- SysMT cego view definitions
--

drop if exists view sysmt_mtview;
create view sysmt_mtview as
select
 mt.mid as mid,
 mt.lockby as lockby,
 srv.srvname as srvname,
 sla.slanr as slanr,
 srv.srvid as srvid,
 sys.sysid as sysid,
 sys.sid as sid,
 sys.mtinfo as mtinfo,
 mt.shortdesc as shortdesc,
 mt.description as description,
 mt.mtfrom as mtfrom,
 mt.mtto as mtto,
 mt.downfrom as downfrom,
 mt.downto as downto,
 mt.status as status,
 mt.ticketnr as ticketnr,
 mt.mttype as mttype,
 mt.thrid as thrid,
 (select tg.tgname as tgname from sysmt_threadgroup tg, sysmt_thread t 
  where ( t.thrid = mt.thrid and t.tgid = tg.tgid )) as tgname,
 (select t.shortdesc as shortdesc from sysmt_thread t where t.thrid = mt.thrid ) as tshortdesc,
 (select count(*) from sysmt_mt mt2 
  where ( ( ( mt2.mtfrom > mt.mtfrom and mt2.mtfrom < mt.mtto ) 
  and mt2.mid != mt.mid ) and mt.sysid = mt2.sysid )) as conflicts
from
 sysmt_mt mt,
 sysmt_service srv,
 sysmt_sla sla,
 sysmt_system sys
where sla.slaid = srv.slaid and sys.srvid = srv.srvid and mt.sysid = sys.sysid;

drop if exists view sysmt_usrview;
create view sysmt_usrview as
select
 u.usrid as usrid,
 u.dbuser as dbuser,
 u.username as username,
 u.userright as userright,
 (select
 count( distinct s.sid)
from
 sysmt_system s,
 sysmt_srv2bgrp p2g,
 sysmt_usr2bgrp u2g
where
 ( ( p2g.srvid = s.srvid
 and
u2g.bgid = p2g.bgid )
  and
 u2g.usrid = u.usrid )
order by
 s.sid) as numsys,
 (select
 count( distinct p.srvid)
from
 sysmt_service p,
 sysmt_srv2bgrp p2g,
 sysmt_usr2bgrp u2g
where
 ( ( p2g.srvid = p.srvid
 and
u2g.bgid = p2g.bgid )
  and
 u2g.usrid = u.usrid )
order by
 p.srvid) as numsrv
from
 sysmt_user u;


drop if exists view sysmt_systemview;
create view sysmt_systemview as
select
 sys.sysid as sysid,
 sys.sid as sid,
 sys.sysrole as sysrole,
 sys.status as status,
 srv.srvtype as srvtype,
 srv.srvname as srvname,
 srv.srvid as srvid,
 sys.sysdesc as description,
 sys.mtinfo as mtinfo
from
 sysmt_system sys,
 sysmt_service srv
where
 sys.srvid = srv.srvid;

drop if exists view sysmt_mtmonthsrv;
create view sysmt_mtmonthsrv as
select
 mt.mid as mid,
 mt.status as status,
 p.srvname as srvname,
 p.srvid as srvid,
 s.sid as sid,
 str2int(date2str(mt.mtfrom,'%m')) as month,
 str2int(date2str(mt.mtfrom,'%Y')) as year
from
 sysmt_mt mt,
 sysmt_system s,
 sysmt_service p
where
 ( mt.sysid = s.sysid
  and
 s.srvid = p.srvid );


drop if exists view sysmt_mtmonthset;
create view sysmt_mtmonthset as
select
 mt.mid as mid,
 mt.status as status,
 ss.setname as setname,
 ss.setid as setid,
 sys.sid as sid,
 str2int(date2str(mt.mtfrom,'%m')) as month,
 str2int(date2str(mt.mtfrom,'%Y')) as year
from
 sysmt_mt mt,
 sysmt_system sys,
 sysmt_sys2set s2s,
 sysmt_systemset ss
where  mt.sysid = s2s.sysid and s2s.sysid = sys.sysid and s2s.setid = ss.setid;


drop if exists view sysmt_sysfwview;
create view sysmt_sysfwview as
select
 sfw.fwrid as fwrid,
 sfw.sysid as sysid,
 (select
 sysmt_system.sid as sid
from
 sysmt_system sysmt_system
where
 sysmt_system.sysid = sfw.sysid
) as sid,
 (select
 sysmt_fwsg.sgname as sgname
from
 sysmt_fwsg sysmt_fwsg
where
 sysmt_fwsg.sgid = sfw.sgidfrom
) as sgfrom,
 (select
 sysmt_fwsg.sgname as sgname
from
 sysmt_fwsg sysmt_fwsg
where
 sysmt_fwsg.sgid = sfw.sgidto
) as sgto,
 (select
 sysmt_fwps.psname as psname
from
 sysmt_fwps sysmt_fwps
where
 sysmt_fwps.psid = sfw.psid
) as portset,
sfw.fwrdesc as fwrdesc
from
 sysmt_sysfw sfw;


drop if exists view sysmt_threadview;
create view sysmt_threadview as
select
 t.thrid as thrid,
 tg.tgname as tgname,
 tg.tgid as tgid,
 t.created as created,
 t.shortdesc as shortdesc,
 t.description as description,
 t.remind as remind,
 bg.bgname as bgname,
 t.status as status,
 u.usrid as usrid,
 t.lastedit as lastedit,
 t.lasteditby as lasteditby,
case when rs.readstat = null then 'UNREAD' else 'READ' end as readstat,
rs.usrid as rsuid,
 case when t.remind < sysdate then 'EXPIRED' else 'INTIME' end as remstat
from
 sysmt_thread t inner join  sysmt_threadgroup tg on t.tgid = tg.tgid
 inner join sysmt_bgroup bg on bg.bgid = t.bgid inner join sysmt_user u on u.usrid = u.usrid
left outer join sysmt_threadstat rs on rs.usrid = u.usrid and rs.thrid = t.thrid;

drop if exists view sysmt_masterhostview;
create view sysmt_masterhostview as
select
 h.hostname as masterhost,
 v2h.vip as vip,
 v2h.status as status
from
 sysmt_vip2host v2h,
 sysmt_host h
where
 v2h.hid = h.hid;

drop if exists view sysmt_mtbyday;
create view sysmt_mtbyday as
select
 sysmt_mt.mid as mid,
 date2str(sysmt_mt.mtfrom,'%d') as day
from
 sysmt_mt sysmt_mt;
drop if exists view sysmt_srv2uid;
create view sysmt_srv2uid as
select
 p.srvname as srvname,
 u2g.usrid as usrid
from
 sysmt_service p,
 sysmt_srv2bgrp p2g,
 sysmt_usr2bgrp u2g
where
 ( p2g.srvid = p.srvid
  and
 u2g.bgid = p2g.bgid );

drop if exists view sysmt_diskview;
create view sysmt_diskview as
select
 d.did as did,
 d.diskname as diskname,
 d.status as status,
 d.scandate as scandate,
 d.disksize as disksize,
 d.diskusage as diskusage,
 d.diskvg as diskvg,
 h.hostname as hostname
from
 sysmt_disk d,
 sysmt_disk2host d2h,
 sysmt_host h
where
 ( d.did = d2h.did
  and
 d2h.hid = h.hid );

drop if exists view sysmt_sid2uid;
create view sysmt_sid2uid as
select
 s.sid as sid,
 u2g.usrid as usrid
from
 sysmt_system s,
 sysmt_srv2bgrp p2g,
 sysmt_usr2bgrp u2g
where
 ( p2g.srvid = s.srvid
  and
 u2g.bgid = p2g.bgid );

drop if exists view sysmt_crewplanview;
create view sysmt_crewplanview as
select
 cp.cpeid as cpeid,
 c.cmid as cmid,
 c.membername as membername,
 cp.activity as activity,
 cp.plannedfrom as plannedfrom,
 cp.plannedto as plannedto,
 cp.mid as mid 
from
 sysmt_crew c,
 sysmt_crewplan cp
where c.cmid = cp.cmid;

drop if exists view sysmt_mtrepview;
create view sysmt_mtrepview as
select
 mt.mid as mid,
 mt.shortdesc as shortdesc,
 p.srvname as srvname,
 s.sid as sid,
 ss.setname as setname,
 mt.shortdesc as shortdesc,
 mt.mtfrom as mtfrom,
 mt.mtto as mtto,
 mt.downfrom as downfrom,
 mt.downto as downto,
 mt.status as status,
 date2str(mt.mtfrom,'%m') as month,
 date2str(mt.mtfrom,'%Y') as year
from
 sysmt_mt mt,
 sysmt_system s,
 sysmt_service p,
 sysmt_sys2set s2s,
 sysmt_systemset ss
where
  mt.sysid = s.sysid and s.srvid = p.srvid and mt.sysid = s2s.sysid and s2s.setid = ss.setid;

drop if exists view sysmt_reqwfsystem;
create view sysmt_reqwfsystem as
select
s.sysid as sysid,
s.sysdesc as sysdesc,
s.sysrole as sysrole,
s.sid as sid,
sap.sapsid as sapsid,
sv.srvname as srvname,
sla.slanr as slanr,
bg.bgname as bgname
from
  sysmt_sla sla,
  sysmt_service sv,
  sysmt_system s,
  sysmt_sapsystem sap,
  sysmt_sysworkflow wf,
  sysmt_bgroup bg
where sla.slaid = sv.slaid and sv.srvid = s.srvid and s.sysid = sap.sysid and s.sysid = wf.sysid and wf.bgid = bg.bgid and wf.status = 'Requested';


-- system specific view

drop if exists view sysmt_sapsystemview;
create view sysmt_sapsystemview as
select
 sys.sysid as sysid,
 sys.sid as sid,
 sapsys.sapsid as sapsid,
 sapsys.dbtype as dbtype,
 sapsys.saptype as saptype
from
 sysmt_system sys,
 sysmt_sapsystem sapsys
where
 sys.sysid = sapsys.sysid;

drop if exists view sysmt_sysresourceview;
create view sysmt_sysresourceview as
select
     s.sid as sid,
     r.resid as resid,	
     ct.comptype as comptype,
     ct.compclass as compclass,
     r.resdesc as resdesc,
     r.resnum as resnum,
     i.sysid as sysid,
     i.instid as instid,
     i.instname as instname,
     r.claimedfrom as claimedfrom,
     r.claimedto as claimedto,
     str2int(date2str(r.claimedfrom,'%m')) as claimedfrommonth,
     str2int(date2str(r.claimedfrom,'%Y')) as claimedfromyear,
     str2int(date2str(r.claimedto,'%m')) as claimedtomonth,
     str2int(date2str(r.claimedto,'%Y')) as claimedtoyear,
     r.monthlyfee as monthlyfee,
     sv.slaid as slaid,
     d.devname as devname
from 
     sysmt_system s inner join sysmt_instance i on s.sysid = i.sysid
     		  inner join sysmt_service sv on s.srvid = sv.srvid
     		  left outer join sysmt_sysresource r on i.instid = r.instid
     		  left outer join sysmt_devicecomp dc on r.cpid = dc.cpid
		  left outer join sysmt_comptype ct on dc.coid = ct.coid
		  left outer join sysmt_device d on d.devid = dc.devid;


drop if exists view sysmt_devicecompview;
create view sysmt_devicecompview as
select
     d.devid as devid,
     d.devname as devname,
     d.devstatus as devstatus,
     c.cpid as cpid,
     ct.compclass as compclass,
     ct.comptype as comptype,
     c.compnum as compnum,
     ( select sum(resnum) + 0 from sysmt_sysresource srv where srv.cpid = c.cpid ) as compallocated
from 
     sysmt_device d,
     sysmt_devicecomp c,
     sysmt_comptype ct
where d.devid = c.devid and c.coid = ct.coid;

drop if exists view sysmt_devicecheckview;
create view sysmt_devicecheckview as
select
     dcv.cpid as cpid,
     dcv.devid as devid,
     dcv.devname as devname,
     dcv.devstatus as devstatus,
     dcv.comptype as comptype,
     dcv.compclass as compclass,
     dcv.compnum as compnum,	
     dcv.compallocated as compallocated,	
     d2s.srvtype as srvtype
from 
     sysmt_devicecompview dcv,
     sysmt_dev2srv d2s
where dcv.devid = d2s.devid;

drop if exists view sysmt_devicestatview;
create view sysmt_devicestatview as
select
     dcv.devid as devid,
     dcv.comptype as comptype,
     dcv.compallocated as compallocated,
     dcv.compnum as comptotal,
     dcv.compnum - dcv.compallocated as compfree,
     dcv.compclass as compclass,
     case when compclass = 'CPU' then 'CPU'
     when compclass = 'MEMORY' then 'MEM'
     when compclass = 'VIRTUAL' then 'VRT'
     when compclass = 'DISK' then 'DSK' else 'NA' end as abvclass
from sysmt_devicecompview dcv order by dcv.compclass, dcv.comptype;

drop if exists view sysmt_deviceserviceview;
create view sysmt_deviceserviceview as
select
     d.devid as devid,
     d.devname as devname,
     d.devtype as devtype,
     d.devstatus as devstatus,	
     d2s.srvtype as srvtype
from 
     sysmt_device d,
     sysmt_dev2srv d2s
where d.devid = d2s.devid;

drop if exists view sysmt_deviceview;
create view sysmt_deviceview as
select
     d.devid as devid,
     d.devname as devname,
     d.devtype as devtype,
     d.devstatus as devstatus,
     d.devserial as devserial,
     r.rackname as rackname,
     l.locname as locname
from 
     sysmt_device d,
     sysmt_rack r,
     sysmt_location l
where d.rckid = r.rckid and r.locid = l.locid;


drop if exists view sysmt_sapinstanceview;
create view sysmt_sapinstanceview as
select 
       i.sysid as sysid,
       i.instid as instid,
       i.instname as instname,
       si.instnr as instnr,
       si.instsmd as instsmd,
       si.instsaptype as instsaptype,
       si.instjavaport as instjavaport,
       si.instdisp as instdisp,
       ( select devname from sysmt_sysresource r, sysmt_devicecomp dc, sysmt_comptype ct, sysmt_device d 
       	 	where r.cpid = dc.cpid and dc.coid = ct.coid and ct.compclass = 'VIRTUAL' and i.instid = r.instid and dc.devid = d.devid ) as insthost
from sysmt_sapinstance si, sysmt_instance i where si.instid = i.instid;

drop if exists view sysmt_sysstatview;
create view sysmt_sysstatview as
select
     s.sysid as sysid,
     s.sid as sid,
     s.srvid as srvid,
     case when status = 'Build' then 'BLD'
     when status = 'Ordered' then 'ORD'
     when status = 'Operation' then 'OPR' else 'NA' end as abvstatus
from sysmt_system s;

drop if exists view sysmt_catalogmoduleview;
create view sysmt_catalogmoduleview as
select
     cm.camid as camid,
     cm.catid as catid,
     ct.compclass as compclass,
     ct.comptype as comptype,
     cm.moddesc as moddesc,
     cm.modnum as modnum,
     cm.modfee as modfee
from sysmt_catalogmodule cm, sysmt_comptype ct where cm.coid = ct.coid;

drop if exists view sysmt_serviceview;
create view sysmt_serviceview as
select 
       srv.srvid as srvid,
       srv.srvname as srvname,
       srv.srvtype as srvtype,
       srv.srvdesc as srvdesc,
       srv.srvmail as srvmail,
       sla.slanr as slanr
from sysmt_service srv, sysmt_sla sla where srv.slaid = sla.slaid;


drop if exists view sysmt_groupview;
create view sysmt_groupview as
select 
       g.bgid as bgid,
       g.bgname as bgname,
       g.bgdesc as bgdesc,
       ( select count(*) from sysmt_srv2bgrp s2g where g.bgid = s2g.bgid ) as numservice,
       ( select count(*) from sysmt_srv2bgrp s2g, sysmt_system s where g.bgid = s2g.bgid and s2g.srvid = s.srvid ) as numsystem
from sysmt_bgroup g;

drop if exists view sysmt_rackview;
create view sysmt_rackview as
select 
       r.rckid as rckid,
       r.rackname as rackname,
       l.locname as locname
from sysmt_rack r, sysmt_location l where r.locid = l.locid;


--
-- SysMT cego procedure definitions
--

drop if exists procedure tsnow;
@
create procedure tsnow() return datetime
begin
        return sysdate;
end;
@

drop if exists procedure str2mny;
@
create procedure str2mny(ms in string(50)) return fixed(2)
begin
   return :ms;
end;
@

drop if exists procedure tstr2ts;
@
create procedure tstr2ts(ts in string(20)) return datetime
begin
	if :ts = null
        then
	   return null;
	else
	   return str2date(:ts, '%d.%m.%Y %H:%M:%S');
	end;
end;
@

drop if exists procedure dstr2ts;
@
create procedure dstr2ts(ds in string(20)) return datetime
begin
	if :ds = null
        then
	   return null;
	else
           return str2date(:ds + ' 00:00:00', '%d.%m.%Y %H:%M:%S');
	end;
end;
@

drop if exists procedure tsgetsec;
@
create procedure tsgetsec(ts in datetime) return int
begin
	var i int;
	:i = str2int(date2str(:ts, '%S'));
	return :i;
end;
@

drop if exists procedure tsgetmin;
@
create procedure tsgetmin(ts in datetime) return int
begin
	var i int;
	:i = str2int(date2str(:ts, '%M'));
	return :i;
end;
@

drop if exists procedure tsgethour;
@
create procedure tsgethour(ts in datetime) return int
begin
	var i int;
	:i = str2int(date2str(:ts, '%H'));
	return :i;
end;
@

drop if exists procedure ts2date;
@
create procedure ts2date(ts in datetime) return string(10)
begin
	return date2str(:ts, '%d.%m.%Y');
end;
@

drop if exists procedure tsgetmonth;
@
create procedure tsgetmonth(ts in datetime) return int
begin
	var i int;
	:i = str2int(date2str(:ts, '%m'));
	return :i;
end;
@

drop if exists procedure tsgetyear;
@
create procedure tsgetyear(ts in datetime) return int
begin
	var i int;
	:i = str2int(date2str(:ts, '%Y'));
	return :i;
end;
@

drop if exists procedure ts2time;
@
create procedure ts2time(ts in datetime) return string(20)
begin
	return date2str(:ts, '%d.%m.%Y %H:%M:%S');
end;
@

drop if exists procedure sladate;
@
create procedure sladate(ts in datetime) return string(20)
begin
	return date2str(:ts, '%m/%Y');
end;
@

drop if exists procedure tsgetyearmonth;
@
create procedure tsgetyearmonth(ts in datetime) return int
begin
	return str2int(date2str(:ts, '%Y%M'));
end;
@

drop if exists procedure getSysId;
@
create procedure getSysId(sid in string(50)) return int
begin
   var sysid int;
   :sysid = (  select sysid as sysid from sysmt_system where sid = :sid );
   return :sysid;
end;
@

drop if exists procedure getSetId;
@
create procedure getSetId(setname in string(100)) return int
begin
   var setid int;
   :setid = ( select setid as setid from sysmt_systemset where setname = :setname );
   return :setid;
end;
@

drop if exists procedure getServiceId;
@
create procedure getServiceId(srvname in string(100)) return int
begin
   var srvid int;
   :srvid = ( select srvid from sysmt_service where srvname = :srvname );
   return :srvid;
end;
@

drop if exists procedure getSrvidForSid;
@
create procedure getSrvidForSid(sid in string(50)) return int
begin
   var srvid int;
   :srvid = ( select srvid from sysmt_system where sid = :sid ); 
   return :srvid;
end;
@

drop if exists procedure getUid;
@
create procedure getUid(dbuser in string(50)) return int
begin
   var uid int;
   :uid = ( select usrid from sysmt_user where dbuser = :dbuser );
   return :uid;
end;
@

drop if exists procedure getBgid;
@
create procedure getBgid(bgname in string(20)) return int
begin
   var bgid int;
   :bgid = ( select bgid from sysmt_bgroup where bgname = :bgname );
   return :bgid;
end;
@

drop if exists procedure newThreadGroup;
@
create procedure newThreadGroup(tgname in string(30),
     tgdesc in string(1000)) return int
begin
   var tgid int;
   :tgid = nextId('TGID');
   insert into sysmt_threadgroup ( tgid, tgname, tgdesc )
    values (:tgid, :tgname, :tgdesc);
   return :tgid;
end;
@

drop if exists procedure newSystemSet;
@
create procedure newSystemSet(setname in string(100),
     description in string(1000)) return int
begin
   var setid int;
   :setid = nextId('SETID');
   insert into sysmt_systemset ( setid, setname, description )
    values (:setid, :setname, :description);
   return :setid;
end;
@

drop if exists procedure newGroup;
@
create procedure newGroup(bgname in string(20),
     bgdesc in string(100)) return int
begin
   var bgid int;
   :bgid = nextId('BGID');
   insert into sysmt_bgroup ( bgid, bgname, bgdesc )
    values (:bgid, :bgname, :bgdesc);
   return :bgid;
end;
@

drop if exists procedure newSystem;
@
create procedure newSystem(sid in string(30),
     srvname in string(20),
     sysdesc in string(1000)) return int
begin
   var srvid int;
   var sysid int;
   var srvtype string(20);

   :sysid = nextId('SYSID');
   :srvid = getServiceId(:srvname);
   :srvtype = ( select srvtype from sysmt_service where srvid = :srvid );

   insert into sysmt_system ( sysid, sid, status, sysdesc, srvid )
    values (:sysid, :sid, 'Ordered', :sysdesc, :srvid);

   if :srvtype = 'SAP'
   then
	insert into sysmt_sapsystem ( sysid ) values ( :sysid );
   elsif :srvtype = 'DB'
   then
    	insert into sysmt_dbsystem ( sysid ) values ( :sysid );
   elsif :srvtype = 'Web'
   then
    	insert into sysmt_websystem ( sysid ) values ( :sysid );
   end;

   return :sysid;
end;
@

drop if exists procedure newThread;
@
create procedure newThread(tgname in string(30),
     shortdesc in string(100),
     bgname in string(20),
     remind in datetime) return int
begin
   var tgid int;
   var thrid int;
   :tgid = (select
 sysmt_threadgroup.tgid as tgid
from
 sysmt_threadgroup sysmt_threadgroup
where
 sysmt_threadgroup.tgname = :tgname
);
   :thrid = nextId('THRID');
   insert into sysmt_thread ( thrid, tgid, shortdesc, created, remind, bgid, status )
   values (:thrid, :tgid, :shortdesc, sysdate, :remind, (select bgid as bgid
                                                         from sysmt_bgroup sysmt_bgroup
                                                         where bgname = :bgname), 'OPEN');
   return :thrid;
end;
@

drop if exists procedure resetApproval;
@
create procedure resetApproval(mid in int)
begin
   var sid string(20);
   var sysid int;
   var srvid int;
   var status string(20);
   var agid int;

   delete from sysmt_mtapproval where mid = :mid;
   delete from sysmt_mtverification where mid = :mid;

   :status = (select sysmt_mtview.status as status from sysmt_mtview where sysmt_mtview.mid = :mid);
   if :status != 'Claimed' and :status != 'Attention'
   then
      :sid = (select sysmt_mtview.sid as sid from sysmt_mtview where sysmt_mtview.mid = :mid);
      :sysid = (select sysmt_mt.sysid as sysid from sysmt_mt where sysmt_mt.mid = :mid);
      :srvid = getSrvidForSid(:sid);
      cursor agCur as select bgid as bgid from sysmt_srv2approval
                      where srvid = :srvid;
      while fetch agCur into (:agid) = true
      begin
            insert into sysmt_mtapproval ( mid, bgid, status ) values (:mid, :agid, 'Requested');
      end;
      close agCur;

      cursor verCur as select bgid as bgid from sysmt_srv2verification where srvid = :srvid;
      while fetch verCur into (:agid) = true
      begin
            insert into sysmt_mtverification ( mid, bgid, status )
             values (:mid, :agid, 'Requested');
      end;
      close verCur;

      update sysmt_mt set status='Requested' where mid = :mid;
   end;
end;
@

drop if exists procedure deleteService;
@
create procedure deleteService(srvid in int)
begin
   var sysid int;

   :sysid = ( select min(sysid) from sysmt_system where srvid = :srvid );
   while :sysid != null
   begin
      deleteSystem(:sysid);
      :sysid = ( select min(sysid) from sysmt_system where srvid = :srvid );
   end;

   delete from sysmt_srv2approval where srvid = :srvid;
   delete from sysmt_srv2bgrp where srvid = :srvid;
   delete from sysmt_service where srvid = :srvid;
end;
@

drop if exists procedure deleteSLA;
@
create procedure deleteSLA(slaid in int)
begin
   var srvid int;

   :srvid = ( select srvid from sysmt_service where slaid = :slaid );
   while :srvid != null
   begin
      deleteService(:srvid);
      :srvid = ( select srvid from sysmt_service where slaid = :slaid );
   end;

   delete from sysmt_sla where slaid = :slaid;
end;
@


drop if exists procedure sysWfStart;
@
create procedure sysWfStart(sysid in int) return int
begin
   var srvid int;
   var seqnr int;
   var wfgid int;
   var wfstatus string(10);
   var wfaction string(100);
   var startdate datetime;

   delete from sysmt_sysworkflow where sysid = :sysid;

   :srvid = (select sysmt_system.srvid as srvid from sysmt_system where sysid = :sysid);

   cursor agCur as
      select bgid, seqnr, wfaction
      from sysmt_srv2workflow
      where srvid = :srvid
      order by seqnr;

   :wfstatus = 'Requested';
   :startdate = sysdate;
   while fetch agCur into (:wfgid, :seqnr, :wfaction) = true
   begin
       if :seqnr = 1
       then 
          :wfstatus = 'Requested';
	  :startdate = sysdate;
       else
          :wfstatus = 'Waiting';
          :startdate = null;
       end;

      insert into sysmt_sysworkflow ( sysid, bgid, seqnr, wfaction, assigned, status )
             values (:sysid, :wfgid, :seqnr, :wfaction, :startdate, :wfstatus);
   end;
   close agCur;

   update sysmt_system set status = 'Build' where sysid = :sysid;

   return 0;
end;
@


drop if exists procedure sysWfFire;
@
create procedure sysWfFire(sysid in int,
     bgname in string(100),
     dbuser in string(50),
     wfcomment in string(500)) return int
begin
   var seqnr int;
   var bgid int;
   var numOpen int;
   var numNext int;

   :bgid = ( select bgid from sysmt_bgroup where bgname = :bgname);
   :seqnr = (select seqnr from sysmt_sysworkflow wf where wf.sysid = :sysid and wf.bgid = :bgid and status = 'Requested');
   update sysmt_sysworkflow set status='Done', released=sysdate, wfcomment=:wfcomment, dbuser=:dbuser
   	  where sysid = :sysid and seqnr = :seqnr and bgid = :bgid;

   :numOpen = ( select count(*) from sysmt_sysworkflow where sysid = :sysid and seqnr = :seqnr and status = 'Requested' );

   if :numOpen = 0
   then
	
	:numNext = ( select count(*) from sysmt_sysworkflow where sysid = :sysid and seqnr = :seqnr + 1);
	if :numNext > 0 
	then
		:seqnr = :seqnr + 1;		
		update sysmt_sysworkflow set status='Requested', assigned=sysdate where sysid = :sysid and seqnr = :seqnr;
	else
		update sysmt_system set status = 'Operation' where sysid = :sysid;
	end;
   end;

   return :seqnr;
end;
@


drop if exists procedure sysWfError;
@
create procedure sysWfError(sysid in int,
     bgname in string(100),
     dbuser in string(50),
     errormsg in string(500)) return int
begin
   var seqnr int;
   var bgid int;
   var numOpen int;
   var numNext int;

   :bgid = ( select bgid from sysmt_bgroup where bgname = :bgname);
   :seqnr = (select sysmt_sysworkflow.seqnr as seqnr 
   	    from sysmt_sysworkflow wf
	    where wf.sysid = :sysid and wf.bgid = :bgid);
   update sysmt_sysworkflow set status='Error', released=sysdate, wfcomment=:errormsg, dbuser=:dbuser
   	  where sysid = :sysid and seqnr = :seqnr and bgid = :bgid;

   return :seqnr;
end;
@

drop if exists procedure sysWfContinue;
@
create procedure sysWfContinue(sysid in int)
begin
   update sysmt_sysworkflow set status='Requested', assigned=sysdate, released=null, dbuser=null, wfcomment=null
   	  where sysid = :sysid and status = 'Error';
end;
@

drop if exists procedure nextId;
@
create procedure nextId(id in string(30)) return int
begin
   var idval int;
   update sysmt_id set idval=idval + 1 where id = :id return :idval = idval;
   return :idval;
end;
@

drop if exists procedure newFWPS;
@
create procedure newFWPS(psname in string(30),
            psdesc in string(500)) return int
begin
   var psid int;
   :psid = nextId('PSID');
   insert into sysmt_fwps ( psid, psname, psdesc )
    values (:psid, :psname, :psdesc);
   return :psid;
end;
@

drop if exists procedure newFileSysTmpl;
@
create procedure newFileSysTmpl(fstmplname in string(50),
     fstmpltype in string(10),
     fstmplrule in string(200),
     fstmpldesc in string(500)) return int
begin
   var tmplid int;
   :tmplid = nextId('TMPLID');
   insert into sysmt_filesystmpl ( tmplid, fstmplname, fstmpltype, fstmplrule, fstmpldesc )
    values (:tmplid, :fstmplname, :fstmpltype, :fstmplrule, :fstmpldesc);
   return :tmplid;
end;
@

drop if exists procedure newAddon;
@
create procedure newAddon(sysid in int,
     aotype in string(20),
     aoname in string(20)) return int
begin
   var aoid int;
   :aoid = nextId('AOID');
   insert into sysmt_addon ( aoid, aotype, aoname, sysid )
    values (:aoid, :aotype, :aoname, :sysid);
   return :aoid;
end;
@

drop if exists procedure newNFS;
@
create procedure newNFS(sysid in int,
     nfspath in string(100),
     nfsnode in string(100),
     nfsmount in string(100)) return int
begin
   var nfsid int;
   :nfsid = nextId('NFSID');
   insert into sysmt_nfs ( nfsid, nfspath, nfsnode, nfsmount, sysid )
    values (:nfsid, :nfspath, :nfsnode, :nfsmount, :sysid);
   return :nfsid;
end;
@

drop if exists procedure newDocument;
@
create procedure newDocument(doctype in string(100),
     docname in string(200)) return int
begin
   var docid int;
   :docid = nextId('DOCID');
   insert into sysmt_document ( docid, doctype, docname )
    values (:docid, :doctype, :docname);
   return :docid;
end;
@

drop if exists procedure newThreadDocument;
@
create procedure newThreadDocument(doctype in string(100),
     docname in string(200),
     thrid in int) return int
begin
   var docid int;
   :docid = newDocument(:doctype, :docname);
   insert into sysmt_doc2thread ( docid, thrid )
    values (:docid, :thrid);
   return :docid;
end;
@

drop if exists procedure newService;
@
create procedure newService(srvname in string(100),
     srvtype in string(100),
     srvdesc in string(1000),
     slanr in string(30)) return int
begin
   var srvid int;
   var slaid int;
   :srvid = nextId('SRVID');
   :slaid = ( select slaid from sysmt_sla  where slanr = :slanr );
   insert into sysmt_service ( srvid, srvname, srvtype, srvdesc, slaid )
    values (:srvid, :srvname, :srvtype, :srvdesc, :slaid);
   return :srvid;
end;
@

drop if exists procedure newUserSearch;
@
create procedure newUserSearch(usname in string(30),
     usspec in string(100),
     dbuser in string(20)) return int
begin
   var usid int;
   var uid int;
   :usid = nextId('USID');
   :uid = getUid(:dbuser);
   insert into sysmt_usersearch ( usid, usname, usspec, usrid )
    values (:usid, :usname, :usspec, :uid);
   return :usid;
end;
@

drop if exists procedure newUser;
@
create procedure newUser(dbuser in string(20),
     username in string(20),
     userdesc in string(1000),
     userright in string(10),
     usermail in string(100)) return int
begin
   var uid int;
   :uid = nextId('UID');
   insert into sysmt_user ( usrid, dbuser, username, userdesc, userright, usermail )
    values (:uid, :dbuser, :username, :userdesc, :userright, :usermail);
   return :uid;
end;
@

drop if exists procedure newFileSysTmplEntry;
@
create procedure newFileSysTmplEntry(tmplid in int,
     fstype in string(50),
     fspath in string(100),
     fsvg in string(100),
     fslv in string(100),
     fsopt in string(100),
     fssizesmall in long,
     fssizebig in long) return int
begin
   var fsid int;
   :fsid = nextId('FSID');
   insert into sysmt_filesystmplentry ( fsid, tmplid, fstype, fspath, fsvg, fslv, fsopt, fssizesmall, fssizebig )
    values (:fsid, :tmplid, :fstype, :fspath, :fsvg, :fslv, :fsopt, :fssizesmall, :fssizebig);
   return :fsid;
end;
@

drop if exists procedure newFWSG;
@
create procedure newFWSG(sgname in string(30),
     sgdesc in string(500)) return int
begin
   var sgid int;
   :sgid = nextId('SGID');
   insert into sysmt_fwsg ( sgid, sgname, sgdesc )
    values (:sgid, :sgname, :sgdesc);
   return :sgid;
end;
@

drop if exists procedure newSysFW;
@
create procedure newSysFW(sgfrom in string(30),
     sgto in string(30),
     portset in string(30),
     fwrdesc in string(500),
     sid in string(20)) return int
begin
   var fwrid int;
   :fwrid = nextId('FWRID');
   insert into sysmt_sysfw ( fwrid, sgidfrom, sgidto, psid, sysid, fwrdesc )
    values (:fwrid, 
    (select sgid from sysmt_fwsg where sgname = :sgfrom ),
    (select sgid from sysmt_fwsg where sgname = :sgto ),
    (select psid from sysmt_fwps where psname = :portset ),
    (select sysid from sysmt_system where sid = :sid),
    :fwrdesc);
   return :fwrid;
end;
@

drop if exists procedure deleteSystem;
@
create procedure deleteSystem(sysid in int)
begin
   var mid int;
   cursor midCur as select mid as mid from sysmt_mt where sysid = :sysid;
   while fetch midCur into ( :mid ) = true
   begin
      delete from sysmt_mtapproval where mid = :mid;
      delete from sysmt_mtlog where mid = :mid;
   end;
   close midCur;

   delete from sysmt_mt where sysid = :sysid;	

   delete from sysmt_sys2set where sysid = :sysid;
   delete from sysmt_sysfw where sysid = :sysid;
   delete from sysmt_syslog where sysid = :sysid;

   delete from sysmt_sysworkflow where sysid = :sysid;

   delete from sysmt_sysresource where instid in ( select instid from sysmt_instance where sysid = :sysid );
   delete from sysmt_sapinstance where instid in ( select instid from sysmt_instance where sysid = :sysid );
   delete from sysmt_instance where sysid = :sysid;
   delete from sysmt_sapsystem where sysid = :sysid;

   delete from sysmt_sapfilesys where sysid = :sysid;
   delete from sysmt_sapaddon where sysid = :sysid;
   delete from sysmt_sapdisp where sysid = :sysid;
   delete from sysmt_sapnfs where sysid = :sysid;

   delete from sysmt_dbsystem where sysid = :sysid;

   delete from sysmt_websystem where sysid = :sysid;

   delete from sysmt_system where sysid = :sysid;
   delete from sysmt_registeredsystem where sysid = :sysid;
end;
@


drop if exists procedure impVip2Host;
@
create procedure impVip2Host(vip in string(30),
     hostname in string(30),
     status in string(15))
begin
   var hid int;
   :hid = getHostId(:hostname);
   if :hid != null
   then
      delete from sysmt_vip2host where ( vip = :vip and hid = :hid );
      insert into sysmt_vip2host ( vip, hid, status )
       values (:vip, :hid, :status);
   end;
end;
@

drop if exists procedure newGroupSearch;
@
create procedure newGroupSearch(gsname in string(30),
     gsspec in string(100)) return int
begin
   var gsid int;
   :gsid = nextId('GSID');
   insert into sysmt_groupsearch ( gsid, gsname, gsspec )
    values (:gsid, :gsname, :gsspec);
   return :gsid;
end;
@

drop if exists procedure newMt;
@
create procedure newMt(sid in string(20),
     shortdesc in string(50),
     description in string(500),
     mtfrom in datetime,
     mtto in datetime,
     downfrom in datetime,
     downto in datetime,
     status in string(10),
     ticketnr in bigint(20),
     mttype in string(10),
     thrid in int) return int
begin
   var mid int;
   var sysid int;
   var srvid int;
   var agid int;
   :mid = nextId('MID');
   :sysid = getSysId(:sid);
   insert into sysmt_mt ( mid, sysid, shortdesc, description, mtfrom, mtto, downfrom, downto, status, ticketnr, mttype, thrid )
    values (:mid, :sysid, :shortdesc, :description, :mtfrom, :mtto, :downfrom, :downto, :status, :ticketnr, :mttype, :thrid);
   :srvid = getSrvidForSid(:sid);
   cursor agCur as select bgid from sysmt_srv2approval where srvid = :srvid;
   while fetch agCur into (:agid) = true
   begin
      insert into sysmt_mtapproval ( mid, bgid, status ) values (:mid, :agid, 'Requested');
   end;
   close agCur;
   return :mid;
end;
@

drop if exists procedure newSLA;
@
create procedure newSLA(slanr in string(30),
     shortdesc in string(100)) return int
begin
   var slaid int;
   :slaid = nextId('SLAID');
   insert into sysmt_sla ( slaid, slanr, shortdesc )
    values (:slaid, :slanr, :shortdesc);
   return :slaid;
end;
@

drop if exists procedure impFS;
@
create procedure impFS(sid in string(20),
     fspath in string(100),
     fssize in long)
begin
   var sysid int;
   :sysid = getSysId(:sid);
   if :sysid != null
   then
      update sysmt_filesys set fssize=:fssize where ( sysid = :sysid
 and
fspath = :fspath );
   end;
end;
@

drop if exists procedure newCrew;
@
create procedure newCrew(membername in string(100),
     description in string(1000)) return int
begin
   var cmid int;
   :cmid = nextId('CMID');
   insert into sysmt_crew ( cmid, membername, description )
    values (:cmid, :membername, :description);
   return :cmid;
end;
@

drop if exists procedure newSkill;
@
create procedure newSkill(skname in string(100),
     skdesc in string(1000)) return int
begin
   var skid int;
   :skid = nextId('SKID');
   insert into sysmt_skill ( skid, skname, skdesc )
    values (:skid, :skname, :skdesc);
   return :skid;
end;
@

drop if exists procedure newCrewPlanEntry;
@
create procedure newCrewPlanEntry(activity in string(200),
     plannedfrom in datetime,
     plannedto in datetime,
     cmid in int,
     mid in int) return int
begin
   var cpeid int;
   :cpeid = nextId('CPEID');
   insert into sysmt_crewplan ( cpeid, activity, plannedfrom, plannedto, cmid, mid )
    values (:cpeid, :activity, :plannedfrom, :plannedto, :cmid, :mid);
   return :cpeid;
end;
@

drop if exists procedure newResourceEntry;
@
create procedure newResourceEntry(activity in string(200),
     resfrom in datetime,
     resto in datetime,
     resid in int,
     mid in int) return int
begin
   var rpeid int;
   :rpeid = nextId('RPEID');
   insert into sysmt_resourceplan ( rpeid, activity, resfrom, resto, resid, mid )
    values (:rpeid, :activity, :resfrom, :resto, :resid, :mid);
   return :rpeid;
end;
@

--
-- SAP specific procedures
--

drop if exists procedure newInstance;
@
create procedure newInstance(sysid in int, instname in string(50)) return int
begin
   var instid int;
   var srvtype string(100);

   :instid = nextId('INSTID');

   insert into sysmt_instance (instid, instname, sysid ) values ( :instid, :instname,  :sysid );	
   
   :srvtype = ( select srvtype from sysmt_systemview where sysid = :sysid );
     
   if :srvtype = 'SAP'
   then
      insert into sysmt_sapinstance (instid ) values ( :instid );
   elsif :srvtype = 'DB'
   then
    	insert into sysmt_dbinstance ( instid ) values ( :instid );
   elsif :srvtype = 'WEB'
   then
    	insert into sysmt_webinstance ( instid ) values ( :instid );
   end;

   return :instid;

end;
@

drop if exists procedure newCatalog;
@
create procedure newCatalog(catname in string(100),
     category in string(20),
     srvtype in string(100),
     catdesc in string(1000),
     catstatus in string(20)) return int
begin
   var catid int;
   :catid = nextId('CATID');
   insert into sysmt_catalog ( catid, catname, category, srvtype, catdesc, catstatus )
    values (:catid, :catname, :category, :srvtype, :catdesc, :catstatus );
   return :catid;
end;
@

drop if exists procedure newLocation;
@
create procedure newLocation(locname in string(100),
       locdesc in string(500)) return int
begin
   var locid int;

   :locid = nextId('LOCID');
   insert into sysmt_location ( locid, locname, locdesc )
    values (:locid, :locname, :locdesc );
   return :locid;
end;
@
 

drop if exists procedure newRack;
@
create procedure newRack(rackname in string(100),
       locname in string(100)) return int
begin
   var rckid int;
   var locid int;

   :rckid = nextId('RCKID');
   :locid = ( select locid from sysmt_location where locname = :locname);
   insert into sysmt_rack ( rckid, rackname, locid )
    values (:rckid, :rackname, :locid );
   return :rckid;
end;
@

drop if exists procedure newDevice;
@
create procedure newDevice(devname in string(100),
     devtype in string(20)) return int
begin
   var devid int;
   :devid = nextId('DEVID');
   insert into sysmt_device ( devid, devname, devtype )
    values (:devid, :devname, :devtype );
   if :devtype = 'VIRTUAL'
   then
	newDeviceComp(:devid, 'VIP', 1);
   end;

   return :devid;
end;
@

drop if exists procedure newDeviceComp;
@
create procedure newDeviceComp(devid in int,
       		 comptype in string(20),
            	 compnum in int) return int
begin
   var cpid int;
   var coid int;
   :cpid = nextId('CPID');
   :coid = ( select coid from sysmt_comptype where comptype = :comptype );
   insert into sysmt_devicecomp ( cpid, devid, coid, compnum )
    values ( :cpid, :devid, :coid, :compnum );
   return :cpid;
end;
@

drop if exists procedure newCompType;
@
create procedure newCompType(comptype in string(20),
     compclass in string(20)) return int
begin
   var coid int;
   :coid = nextId('COID');
   insert into sysmt_comptype ( coid, comptype, compclass )
   	  values ( :coid, :comptype, :compclass );
   return :coid;
end;
@

drop if exists procedure deleteCompType;
@
create procedure deleteCompType(coid in int )
begin
   delete from sysmt_devicecomp where coid = :coid;
   delete from sysmt_comptype where coid = :coid;
end;
@

drop if exists procedure deleteCatalog;
@
create procedure deleteCatalog(catid in int)
begin
   delete from sysmt_catalogmodule where catid = :catid;
   delete from sysmt_catalog where catid = :catid;
end;
@

drop if exists procedure newOrder;
@
create procedure newOrder(ordby in string(50),
     orddesc in string(100)) return int
begin
   var ordid int;
   :ordid = nextId('ORDID');
   insert into sysmt_orders ( ordid, orddate, orddesc, ordby )
    values (:ordid, sysdate, :orddesc, :ordby );
   return :ordid;
end;
@

drop if exists procedure newResource;
@
create procedure newResource(cpid in int,
       resnum in int,
       resdesc in string(100),
       instid in int,
       ordid in int,
       claimedfrom in datetime,
       claimedto in datetime,
       monthlyfee in fixed(2)) return int
begin
	var resid int;
	:resid = nextId('RESID');
	
      	insert into sysmt_sysresource ( resid, cpid, resnum, resdesc, instid, ordid, claimedfrom, claimedto, monthlyfee ) 
	values ( :resid, :cpid, :resnum, :resdesc, :instid, :ordid, :claimedfrom, :claimedto, :monthlyfee);
   	return :resid;
end;
@

drop if exists procedure deleteDevice;
@
create procedure deleteDevice(devid in int)
begin
   var cpid int;
   cursor cpCur as select cpid as cpid from sysmt_devicecomp where devid = :devid;
   while fetch cpCur into (:cpid) = true
   begin
	   delete from sysmt_sysresource where cpid = :cpid;	
   end;
   close cpCur;

   delete from sysmt_dev2srv where devid = :devid;
   delete from sysmt_devicecomp where devid = :devid;
   delete from sysmt_device where devid = :devid;

end;
@

drop if exists procedure deleteGroup;
@
create procedure deleteGroup(bgid in int )
begin
   delete from sysmt_srv2approval where bgid = :bgid;
   delete from sysmt_srv2bgrp where bgid = :bgid;
   delete from sysmt_srv2notification where bgid = :bgid;
   delete from sysmt_srv2workflow where bgid = :bgid;
   delete from sysmt_srv2verification where bgid = :bgid;
   delete from sysmt_set2bgrp where bgid = :bgid;
   delete from sysmt_sysworkflow where bgid = :bgid;
   delete from sysmt_tg2bgrp where bgid = :bgid;
   delete from sysmt_usr2bgrp where bgid = :bgid;
   delete from sysmt_gs2bgrp where bgid = :bgid;
   delete from sysmt_mtapproval where bgid = :bgid;
   delete from sysmt_mtverification where bgid = :bgid;
   delete from sysmt_bgright where bgid = :bgid;
   update sysmt_thread set bgid = null where bgid = :bgid;
   delete from sysmt_bgroup where bgid = :bgid;
end;
@

drop if exists procedure deleteUser;
@
create procedure deleteUser(uid in int )
begin
   delete from sysmt_threadstat where usrid = :uid;
   delete from sysmt_usr2bgrp where usrid = :uid;
   delete from sysmt_usersearch where usrid = :uid;
   delete from sysmt_custcond where usrid = :uid;
   delete from sysmt_user where usrid = :uid;
end;
@

drop if exists procedure newCatalogModule;
@
create procedure newCatalogModule(catid in int,
       comptype in string(20),
       moddesc in string(100),
       modnum in int,
       modfee in fixed(2)) return int
begin
   var camid int;
   var coid int;

   :camid = nextId('CAMID');
   :coid = ( select coid from sysmt_comptype where comptype = :comptype);

   insert into sysmt_catalogmodule ( camid, catid, coid, moddesc, modnum, modfee )
   	  values ( :camid, :catid, :coid, :moddesc, :modnum, :modfee );
   return :camid;
end;
@


drop if exists procedure mtApprove;
@
create procedure mtApprove(mid in int, bgname in string(100))
begin
	var bgid int;
	var numOpen int;

	:bgid = ( select bgid from sysmt_bgroup where bgname = :bgname );
	update sysmt_mtapproval set status = 'Approved' where mid = :mid and bgid = :bgid;

	:numOpen = ( select count(*) from sysmt_mtapproval where mid = :mid and status = 'Requested' );

	if :numOpen = 0
	then
		update sysmt_mt set status = 'Approved' where mid = :mid;
	end;
end;
@

drop if exists procedure mtVerify;
@
create procedure mtVerify(mid in int, bgname in string(100))
begin
	var bgid int;
	var numOpen int;

	:bgid = ( select bgid from sysmt_bgroup where bgname = :bgname );
	update sysmt_mtverification set status = 'Verified' where mid = :mid and bgid = :bgid;

	:numOpen = ( select count(*) from sysmt_mtverification where mid = :mid and status = 'Requested' );

	if :numOpen = 0
	then
		update sysmt_mt set status = 'Verified' where mid = :mid;
	end;
end;
@

drop if exists procedure cloneSid;
@
create procedure cloneSid(mid in int, sid in string(50))
begin

   var shortdesc string(100);
   var description string(500);
   var mtfrom datetime;
   var mtto datetime;   
   var downfrom datetime;
   var downto datetime;
   var mttype string(50);
   var thrid int;
   var clonemid int;

   cursor cloneCur as select shortdesc, description, mtfrom, mtto, downfrom, downto, mttype, thrid from sysmt_mt where mid = :mid;

   fetch cloneCur into (:shortdesc, :description, :mtfrom, :mtto, :downfrom, :downto, :mttype, :thrid );
   close cloneCur;

   :clonemid = newMt(:sid, :shortdesc, :description, :mtfrom, :mtto, :downfrom, :downto, 'Requested', 0, :mttype, :thrid);   

end;
@

drop if exists procedure cloneSet;
@
create procedure cloneSet(mid in int, setname in string(100))
begin

   var shortdesc string(100);
   var description string(500);
   var mtfrom datetime;
   var mtto datetime;   
   var downfrom datetime;
   var downto datetime;
   var mttype string(50);
   var thrid int;
   var clonemid int;

   var sid string(50);
   var mastersid string(50);

   cursor cloneCur as select sid, shortdesc, description, mtfrom, mtto, downfrom, downto, mttype, thrid from sysmt_mtview where mid = :mid;
   cursor setCur as select sid from sysmt_system s, sysmt_sys2set s2s, sysmt_systemset ss where s.sysid = s2s.sysid 
   	  	    	       	    and s2s.setid = ss.setid and ss.setname = :setname;

   fetch cloneCur into (:mastersid, :shortdesc, :description, :mtfrom, :mtto, :downfrom, :downto, :mttype, :thrid );
   close cloneCur;

   while fetch setCur into ( :sid ) = true
   begin
	if :sid != :mastersid
	then
	  :clonemid = newMt(:sid, :shortdesc, :description, :mtfrom, :mtto, :downfrom, :downto, 'Requested', 0, :mttype, :thrid);   
        end;
   end;

   close setCur;

end;
@

drop if exists procedure deleteThreadGroup;
@
create procedure deleteThreadGroup(tgid in int )
begin
   delete from sysmt_thread where tgid = :tgid;
   delete from sysmt_tg2bgrp where tgid = :tgid;
   delete from sysmt_threadgroup where tgid = :tgid;
end;
@

drop if exists procedure deleteSystemSet;
@
create procedure deleteSystemSet(setid in int )
begin
   delete from sysmt_sys2set where setid = :setid;
   delete from sysmt_set2bgrp where setid = :setid;
   delete from sysmt_systemset where setid = :setid;
end;
@

drop if exists procedure newSchedule;
@
create procedure newSchedule(schedentry in string(200)) return int
begin
   var schedid int;
   :schedid = nextId('SCHEDID');
   insert into sysmt_schedule ( schedid, schedentry )
    values (:schedid, :schedentry);
   return :schedid;
end;
@
-- base data

-- system addon id
insert into sysmt_id values ( 'AOID', 10000);
-- business group id
insert into sysmt_id values ( 'BGID', 10000);
-- disk id
insert into sysmt_id values ( 'DID', 10000);
-- document id
insert into sysmt_id values ( 'DOCID', 10000);
-- filesystem id
insert into sysmt_id values ( 'FSID', 10000);
-- group search id
insert into sysmt_id values ( 'GSID', 10000);
-- host id
insert into sysmt_id values ( 'HID', 10000);
-- instance id
insert into sysmt_id values ( 'INSTID', 10000);
-- maintenance id
insert into sysmt_id values ( 'MID', 10000);
-- nfs id
insert into sysmt_id values ( 'NFSID', 10000);
-- service id
insert into sysmt_id values ( 'SRVID', 10000);
-- fw port set id
insert into sysmt_id values ( 'PSID', 10000);
-- fw rule set id
insert into sysmt_id values ( 'RSID', 10000);
-- system set id
insert into sysmt_id values ( 'SETID', 10000);
-- fw server group id
insert into sysmt_id values ( 'SGID', 10000);
-- SLA id
insert into sysmt_id values ( 'SLAID', 10000);
-- system id
insert into sysmt_id values ( 'SYSID', 10000);
-- thread group id
insert into sysmt_id values ( 'TGID', 10000);
-- thread id
insert into sysmt_id values ( 'THRID', 10000);
-- fs tmpl id
insert into sysmt_id values ( 'TMPLID', 10000);
-- user id
insert into sysmt_id values ( 'UID', 10000);
-- user search
insert into sysmt_id values ( 'USID', 10000);

-- sys resource 
insert into sysmt_id values ( 'RESID', 10000);

-- crew member 
insert into sysmt_id values ( 'CMID', 10000);

-- skill 
insert into sysmt_id values ( 'SKID', 10000);

-- crew plan entry
insert into sysmt_id values ( 'CPEID', 10000);

-- firewall rule 
insert into sysmt_id values ( 'FWRID', 10000);

-- catalog entry
insert into sysmt_id values ( 'CATID', 10000);

-- device entry
insert into sysmt_id values ( 'DEVID', 10000);

-- order entry
insert into sysmt_id values ( 'ORDID', 10000);

-- component entry
insert into sysmt_id values ( 'CPID', 10000);

-- component type entry
insert into sysmt_id values ( 'COID', 10000);

-- catalog module entry
insert into sysmt_id values ( 'CAMID', 10000);

-- rack id entry
insert into sysmt_id values ( 'RCKID', 10000);

-- location id entry
insert into sysmt_id values ( 'LOCID', 10000);

-- schedule id entry
insert into sysmt_id values ( 'SCHEDID', 10000);


-- static component types
insert into sysmt_comptype values ( 10, 'VIRTUAL', 'VIP');


-- sysparams
insert into sysmt_sysparam values ( 'SCHEMAVERSION', '4.4');
insert into sysmt_sysparam values ( 'SYSMSG', 'Welcome to SysMT');
insert into sysmt_sysparam values ( 'HEADMSG', 'SysMT Version 4.4.0');

insert into sysmt_sysparam values ( 'XBUSER', 'lemkeb');
insert into sysmt_sysparam values ( 'XBPWD', 'init13');
insert into sysmt_sysparam values ( 'XBINT', '10');

insert into sysmt_sysparam values ( 'XBWSDL', 'https://support.syslink.ch/demo/ws/xandria_webservice.wsdl');
insert into sysmt_sysparam values ( 'XBURI', 'http://www.syslink.ch/2013/xandria/webservice');
insert into sysmt_sysparam values ( 'XBLOCAL', 'XandriaWebServiceService');

insert into sysmt_sysparam values ( 'XBMODE', 'OFF');
insert into sysmt_sysparam values ( 'XBMSG', 'XXX');

insert into sysmt_sysparam values ( 'SCHEDSTAT', 'XXX');

insert into sysmt_user ( usrid, dbuser, password, hostinfo, username, userdesc, userright, usermail ) values ( 0, 'admin', '205qjDWdAOD5Q', 'OFFLINE', 'Administrator', 'Admin User', 'admin', 'nomail');
insert into sysmt_user ( usrid, dbuser, password, hostinfo, username, userdesc, userright, usermail ) values ( 1, 'lemke', 'VxlvSgomYNe8s', 'OFFLINE', 'Björn Lemke', 'Admin User', 'admin', 'nomail');

-- to have some test data
insert into sysmt_bgroup values ( 1, 'Group-A', 'Demo Group');
insert into sysmt_bgright values ( 1, 'sysread' );
insert into sysmt_bgright values ( 1, 'syswrite' );
insert into sysmt_bgright values ( 1, 'mtread' );
insert into sysmt_bgright values ( 1, 'mtwrite' );
insert into sysmt_bgright values ( 1, 'catread' );
insert into sysmt_bgright values ( 1, 'catwrite' );

insert into sysmt_usr2bgrp values ( 1, 1);

insert into sysmt_bgroup values ( 2, 'Xandria', 'Xandria Demo Group');
insert into sysmt_bgright values ( 2, 'sysread' );
insert into sysmt_bgright values ( 2, 'syswrite' );


insert into sysmt_sla values ( 1, 'SLA-Lemke-IT', 'Demo Data');

insert into sysmt_service ( srvid, srvname, srvtype, srvdesc, srvmail, slaid )values ( 1, 'Lemke-IT', 'SAP', 'Demo Data', 'Demo Data', 1);
insert into sysmt_srv2bgrp values ( 1, 1);
insert into sysmt_srv2bgrp values ( 1, 2);


-- sample component types
insert into sysmt_comptype values ( 100, 'CPU', 'core-i5');
insert into sysmt_comptype values ( 101, 'CPU', 'power7');
insert into sysmt_comptype values ( 102, 'MEMORY', '1GB-DDR3');
insert into sysmt_comptype values ( 103, 'MEMORY', '4GB-DDR3');
insert into sysmt_comptype values ( 104, 'DISK', '100GB-RAID10');
insert into sysmt_comptype values ( 105, 'DISK', '100GB-RAID5');

insert into sysmt_location values ( 1000, 'Generic Datacenter', 'Generic Datacenter');
insert into sysmt_rack values ( 1000, 'Generic Rack', 1000);
 

