///////////////////////////////////////////////////////////////////////////////
//                                                         
// MySQLFormater.cc  
// ----------------                                                        
// Formatting MySQL SQL statements
//
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2015 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: MySQLFormater
// 
// Description: 
//
//
///////////////////////////////////////////////////////////////////////////////

#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "MySQLFormater.h"

#include "cego/CegoXMLdef.h"
#include "cego/CegoAction.h"
#include "cego/CegoProcedure.h"
#include "cego/CegoQuery.h"

#include "lfcbase/BigInteger.h"
#include "lfcbase/BigDecimal.h"
#include "lfcbase/Datetime.h"

#define INDENT Chain(" ")
#define MYSQLDATEFORMAT "%Y-%m-%d %H:%M:%S"
#define ROWCOUNTFLUSH 1000

#define COUNTERTAB_DEF "DROP TABLE IF EXISTS COUNTERTAB;\n\
CREATE TABLE COUNTERTAB (l_id VARCHAR(30) NOT NULL, l_val INTEGER, PRIMARY KEY (l_id));\n"

#define SETCOUNT_PROC "DROP PROCEDURE IF EXISTS SETCOUNT\n\
//\
CREATE PROCEDURE SETCOUNT (l_id VARCHAR(30), l_val INTEGER) DETERMINISTIC\n\
BEGIN\n\
        UPDATE COUNTERTAB SET idval = l_val where id = l_id;\n\
END;\n\
//\n"

#define NEXTCOUNT_PROC "DROP FUNCTION IF EXISTS NEXTCOUNT\n\
//\n\
CREATE FUNCTION NEXTCOUNT (l_id VARCHAR(30)) RETURNS INT DETERMINISTIC\n\
BEGIN\n\
        DECLARE l_val INT;\n\
        UPDATE COUNTERTAB SET idval = LAST_INSERT_ID(idval + 1) where id = l_id;\n\
        SELECT LAST_INSERT_ID() INTO l_val;\n\
        RETURN l_val;\n\
END;\n\
//\n"

MySQLFormater::MySQLFormater(CegoNet *pCegoNet, const Chain& sqlfile) : CegoDatabaseFormater()
{
    if ( pCegoNet )
    {
	_pCegoNet = pCegoNet;
	_pDF = new File(sqlfile);
	_pDF->open(File::WRITE);
    }
    else
    {
	throw Exception(EXLOC, "Cego database connection required for export"); 
    }
}

MySQLFormater::~MySQLFormater()  
{
    _pDF->close();
    delete _pDF;	
}

void MySQLFormater::exportTable()
{
    _pDF->writeChain(Chain(COUNTERTAB_DEF));
    
    long res;
    res = _pCegoNet->doQuery(Chain("list table;"));
    
    ListT<CegoField> schema;
    _pCegoNet->getSchema(schema);
    
    ListT<Chain> tableList;
    ListT<CegoFieldValue> fvl;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {

	CegoFieldValue *pFV = fvl.First();
	Chain tableName = pFV->valAsChain();
	
	tableList.Insert(tableName);
	fvl.Empty();	
    }
    
    Chain *pTable = tableList.First();
    while ( pTable ) 
    {
	cout << "   Table " << *pTable;
	res = _pCegoNet->doQuery(Chain("desc table ") + *pTable + Chain(";"));

	Chain dropStmt;

	dropStmt=Chain("DROP TABLE IF EXISTS ") + *pTable + Chain(";\n");
	_pDF->writeChain(dropStmt);
	
	Chain createStmt=Chain("CREATE TABLE ") + *pTable + Chain("(");

	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);
    
	ListT<CegoFieldValue> fvl;
	bool isFirst=true;
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{
	    if ( isFirst == false )
		createStmt += Chain(", ");
	    isFirst=false;

	    CegoFieldValue *pFV = fvl.First();
	    Chain attrName = pFV->valAsChain();
	    pFV = fvl.Next();
	    Chain attrType = pFV->valAsChain();
	    pFV = fvl.Next();
	    Chain attrSize = pFV->valAsChain();
	    pFV = fvl.Next();
	    Chain attrDef = pFV->valAsChain();
	    if ( attrDef == Chain("sysdate") )
		attrDef = Chain("CURRENT_TIMESTAMP");
	    pFV = fvl.Next();
	    Chain attrNull = pFV->valAsChain();

	    createStmt += attrName + Chain(" ");
	    
	    if ( attrType == Chain(XML_STRING_VALUE) )
		createStmt += Chain("VARCHAR") + Chain("(") + attrSize + Chain(")");
	    else if ( attrType == Chain(XML_INT_VALUE) )
		createStmt += Chain("INT");
	    else if ( attrType == Chain(XML_LONG_VALUE) )
		createStmt += Chain("LONG");
	    else if ( attrType == Chain(XML_FLOAT_VALUE) )
		createStmt += Chain("FLOAT");
	    else if ( attrType == Chain(XML_DOUBLE_VALUE) )
		createStmt += Chain("DOUBLE");
	    else if ( attrType == Chain(XML_DECIMAL_VALUE) )
		createStmt += Chain("DECIMAL(10,") + attrSize + Chain(")");	    
	    else if ( attrType == Chain(XML_BOOL_VALUE) )
		createStmt += Chain("BOOLEAN");
	    else if ( attrType == Chain(XML_DATETIME_VALUE) )
		createStmt += Chain("DATETIME");
	    else if ( attrType == Chain(XML_FIXED_VALUE) )
		createStmt += Chain("DECIMAL(10,") + attrSize + Chain(")");
	    else if ( attrType == Chain(XML_BIGINT_VALUE) )
		createStmt += Chain("DECIMAL(") + attrSize + Chain(")");
	    else if ( attrType == Chain(XML_SMALLINT_VALUE) )
		createStmt += Chain("SMALLINT");
	    else if ( attrType == Chain(XML_TINYINT_VALUE) )
		createStmt += Chain("TINYINT");
	    else if ( attrType == Chain(XML_BLOB_VALUE) )
		createStmt += Chain("BLOB");
	    else if ( attrType == Chain(XML_CLOB_VALUE) )
		createStmt += Chain("TEXT");
	   	    
	    if ( attrDef != Chain(XML_NULL_VALUE))
	    {
		if ( attrType == Chain(XML_INT_VALUE)
		     || attrType == Chain(XML_LONG_VALUE)
		     || attrType == Chain(XML_DOUBLE_VALUE)		     
		     || attrType == Chain(XML_FIXED_VALUE)
		     || attrType == Chain(XML_DECIMAL_VALUE)
		     || attrType == Chain(XML_BIGINT_VALUE)		     
		     || attrType == Chain(XML_SMALLINT_VALUE)
		     || attrType == Chain(XML_TINYINT_VALUE)
		    )
		{		     
		    createStmt += Chain(" DEFAULT ") + attrDef;
		}
		else
		{
		    createStmt += Chain(" DEFAULT '") + attrDef + Chain("'");
		}
	    }
	    if ( attrNull == Chain("no"))
		createStmt += Chain(" NOT NULL");
	    fvl.Empty();	
	}

	createStmt += Chain(");\n");
	_pDF->writeChain(createStmt);


	// dump table content
	res = _pCegoNet->doQuery(Chain("select * from ") + *pTable + Chain(";"));
	    
	schema.Empty();
	fvl.Empty();
	
	_pCegoNet->getSchema(schema);

	long rowCount=0;
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{

	    if ( rowCount > 0 && rowCount % ROWCOUNTFLUSH == 0 )
	    {
		cout << " ... " << rowCount;
		cout.flush();
	    }
	    
	    Chain insertStmt=Chain("INSERT INTO ") + *pTable + Chain(" VALUES (");
	    
	    CegoFieldValue *pFV = fvl.First();
	    while ( pFV )
	    {
		Chain val = pFV->dbFormat(this);
		insertStmt += val;
		pFV = fvl.Next();
		
		if ( pFV )
		    insertStmt += Chain(", ");
	    }
	    insertStmt += Chain(");\n");
	    _pDF->writeChain(insertStmt);
	    fvl.Empty();

	    rowCount++;
	    
	}

	cout << " ... " << rowCount << " rows exported" << endl;

	pTable = tableList.Next();
    }
}

void MySQLFormater::exportProcedure()
{
    
    long res;
    res = _pCegoNet->doQuery(Chain("list procedure;"));
    
    ListT<CegoField> schema;
    _pCegoNet->getSchema(schema);
    
    ListT<Chain> procList;
    ListT<CegoFieldValue> fvl;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {
	CegoFieldValue *pFV = fvl.First();
	Chain procName = pFV->valAsChain();
	procList.Insert(procName);
	fvl.Empty();	
    }

    _pDF->writeChain(Chain("DELIMITER //\n"));

    _pDF->writeChain(Chain(SETCOUNT_PROC));
    _pDF->writeChain(Chain(NEXTCOUNT_PROC));
    
    Chain *pProcName = procList.First();
    while ( pProcName ) 
    {

	cout << "   Procedure " << *pProcName;
	
	res = _pCegoNet->doQuery(Chain("show procedure ") + *pProcName + Chain(";"));

	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);
    
	ListT<CegoFieldValue> fvl;
	Chain procText;
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{
	    CegoFieldValue *pFV = fvl.First();
	    procText = pFV->valAsChain();    
	    fvl.Empty();	
	}
	
	CegoAction* pA = new CegoAction(0,0);
	pA->cleanUp();

	/*
	Chain procLoad = Chain("load procedure copytab(x in int, y in string(30)) return int \
begin \
   var a int; \
   :a = ( select b from cXX ); \
   insert into t2 select a, b from t1; \
   return :a; \
end;");
	*/

	Chain procLoad = Chain("load ") + procText;
	pA->setCommandChain(procLoad);
	pA->parse();

	CegoProcedure* pProc = pA->getProcedure();

	Chain dropStmt;
	if ( pProc->getProcType() == CegoProcedure::FUNCTION )
	    dropStmt=Chain("DROP FUNCTION IF EXISTS ") + *pProcName + Chain("\n");
	else
	    dropStmt=Chain("DROP PROCEDURE IF EXISTS ") + *pProcName + Chain("\n");
	
	_pDF->writeChain(dropStmt);
	_pDF->writeChain(Chain("//\n"));
	_pDF->writeChain(pProc->dbFormat(this));
	_pDF->writeChain(Chain("//\n"));

	delete pA;

	cout << " ... ok" << endl;
	pProcName = procList.Next();
    }

}


void MySQLFormater::exportView()
{
    long res;
    res = _pCegoNet->doQuery(Chain("list view;"));
    
    ListT<CegoField> schema;
    _pCegoNet->getSchema(schema);
    
    ListT<Chain> viewList;
    ListT<CegoFieldValue> fvl;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {
	CegoFieldValue *pFV = fvl.First();
	Chain viewName = pFV->valAsChain();
	viewList.Insert(viewName);
	fvl.Empty();	
    }
    
    Chain *pViewName = viewList.First();
    while ( pViewName ) 
    {

	cout << "   View " << *pViewName;
		
	res = _pCegoNet->doQuery(Chain("show view ") + *pViewName + Chain(";"));

	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);
    
	ListT<CegoFieldValue> fvl;
	Chain viewText;
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{
	    CegoFieldValue *pFV = fvl.First();
	    viewText = pFV->valAsChain();    
	    fvl.Empty();	
	}
	
	CegoAction* pA = new CegoAction(0,0);
	pA->cleanUp();

	Chain viewLoad = Chain("load ") + viewText;
	pA->setCommandChain(viewLoad);
	pA->parse();


	CegoSelect* pSelect = pA->getSelect();
		
	Chain dropStmt;
       
	dropStmt=Chain("DROP VIEW IF EXISTS ") + *pViewName + Chain(";\n");
	_pDF->writeChain(dropStmt);

	Chain createStmt = Chain("CREATE VIEW ") + *pViewName + Chain(" AS ");
	createStmt += pSelect->dbFormat(this);
	createStmt += Chain(";\n");
	

	_pDF->writeChain(createStmt);
	delete pA;

	cout << " ... ok" << endl;
	pViewName = viewList.Next();
    }
    
}

void MySQLFormater::exportIndex()
{
    long res;
    res = _pCegoNet->doQuery(Chain("list avltree;"));
        
    ListT<CegoField> schema;
    _pCegoNet->getSchema(schema);
    
    ListT<Chain> indexList;
    ListT<CegoFieldValue> fvl;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {
	CegoFieldValue *pFV = fvl.First();
	Chain indexName = pFV->valAsChain();
	indexList.Insert(indexName);
	fvl.Empty();	
    }
    
    Chain *pIndex = indexList.First();
    while ( pIndex ) 
    {

	cout << "   Index " << *pIndex;
		
	res = _pCegoNet->doQuery(Chain("desc index ") + *pIndex + Chain(";"));

	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);
    
	Chain tableName;
	Chain indexType;
	ListT<Chain> attrList;
	
	ListT<CegoFieldValue> fvl;	
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{	    
	    CegoFieldValue *pFV = fvl.First();
	    Chain attrName = pFV->valAsChain();
	    attrList.Insert(attrName);
	    pFV = fvl.Next();
	    tableName = pFV->valAsChain();
	    pFV = fvl.Next();
	    indexType = pFV->valAsChain();

	    fvl.Empty();
	}

	Chain cmd;


	if ( indexType == Chain(XML_PINDEX_VALUE) )
	{
	    //cmd=Chain("ALTER TABLE ") + tableName + Chain(" DROP PRIMARY KEY;\n");
	    // _pDF->writeChain(cmd);
	    cmd=Chain("ALTER TABLE ") + tableName + Chain(" ADD PRIMARY KEY ") + Chain("(");
	}
	else if ( indexType == Chain(XML_UINDEX_VALUE) )
	{
	    // cmd=Chain("DROP INDEX IF EXISTS ") + *pIndex + Chain(";\n");
	    // _pDF->writeChain(cmd);
	    
	    cmd=Chain("CREATE UNIQUE INDEX ") + *pIndex + Chain(" ON ") + tableName + Chain("(");
	}
	else
	{
	    // cmd=Chain("DROP INDEX IF EXISTS ") + *pIndex + Chain(";\n");
	    // _pDF->writeChain(cmd);
	    
	    cmd=Chain("CREATE INDEX ") + *pIndex + Chain(" ON ") + tableName + Chain("(");
	}
	Chain *pAttr = attrList.First();
	bool isFirst=true;
	while ( pAttr )
	{
	    if ( isFirst == false )
		cmd += Chain(",");
	    isFirst=false;

	    cmd += *pAttr;
	    pAttr = attrList.Next();
	}
	
	cmd += Chain(");\n");
	_pDF->writeChain(cmd);


	cout << " ... ok" << endl;
	pIndex = indexList.Next();
    }

    
}

void MySQLFormater::exportBtree()
{
    long res;
    res = _pCegoNet->doQuery(Chain("list btree;"));
        
    ListT<CegoField> schema;
    _pCegoNet->getSchema(schema);
    
    ListT<Chain> indexList;
    ListT<CegoFieldValue> fvl;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {
	CegoFieldValue *pFV = fvl.First();
	Chain indexName = pFV->valAsChain();
	indexList.Insert(indexName);
	fvl.Empty();	
    }
    
    Chain *pIndex = indexList.First();
    while ( pIndex ) 
    {

	cout << "   Btree " << *pIndex;
		
	res = _pCegoNet->doQuery(Chain("desc btree ") + *pIndex + Chain(";"));

	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);
    
	Chain tableName;
	Chain indexType;
	ListT<Chain> attrList;
	
	ListT<CegoFieldValue> fvl;	
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{	    
	    CegoFieldValue *pFV = fvl.First();
	    Chain attrName = pFV->valAsChain();
	    attrList.Insert(attrName);
	    pFV = fvl.Next();
	    tableName = pFV->valAsChain();
	    pFV = fvl.Next();
	    indexType = pFV->valAsChain();

	    fvl.Empty();
	}

	Chain cmd;


	if ( indexType == Chain(XML_PINDEX_VALUE) )
	{
	    // cmd=Chain("ALTER TABLE ") + tableName + Chain(" DROP PRIMARY KEY;\n");
	    // _pDF->writeChain(cmd);
	    cmd=Chain("ALTER TABLE ") + tableName + Chain(" ADD PRIMARY KEY ") + Chain("(");
	}
	else if ( indexType == Chain(XML_UINDEX_VALUE) )
	{
	    // cmd=Chain("DROP INDEX IF EXISTS ") + *pIndex + Chain(";\n");
	    // _pDF->writeChain(cmd);
	    
	    cmd=Chain("CREATE UNIQUE INDEX ") + *pIndex + Chain(" ON ") + tableName + Chain("(");
	}
	else
	{
	    // cmd=Chain("DROP INDEX IF EXISTS ") + *pIndex + Chain(";\n");
	    // _pDF->writeChain(cmd);
	    
	    cmd=Chain("CREATE INDEX ") + *pIndex + Chain(" ON ") + tableName + Chain("(");
	}
	
	Chain *pAttr = attrList.First();
	bool isFirst=true;
	while ( pAttr )
	{
	    if ( isFirst == false )
		cmd += Chain(",");
	    isFirst=false;

	    cmd += *pAttr;
	    pAttr = attrList.Next();
	}
	
	cmd += Chain(");\n");
	_pDF->writeChain(cmd);

	cout << " ... ok" << endl;
	pIndex = indexList.Next();
    }

    
}


void MySQLFormater::exportConstraint()
{
    
    long res;
    res = _pCegoNet->doQuery(Chain("list key;"));
        
    ListT<CegoField> schema;
    _pCegoNet->getSchema(schema);
    
    ListT<Chain> keyList;
    ListT<CegoFieldValue> fvl;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {
	CegoFieldValue *pFV = fvl.First();
	Chain keyName = pFV->valAsChain();
	keyList.Insert(keyName);
	fvl.Empty();	
    }
    
    Chain *pKey = keyList.First();
    while ( pKey ) 
    {

	cout << "   Constraint " << *pKey;
	
	res = _pCegoNet->doQuery(Chain("desc key ") + *pKey + Chain(";"));
	
	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);

	Chain keyTable;
	Chain refTable;
	ListT<Chain> keyAttrList;
	ListT<Chain> refAttrList;

	ListT<CegoFieldValue> fvl;
	while ( _pCegoNet->fetchData(schema, fvl)  )
	{
	    
	    CegoFieldValue *pFV = fvl.First();

	    Chain tableName = pFV->valAsChain();
	    pFV = fvl.Next();
	    Chain attrName = pFV->valAsChain();
	    pFV = fvl.Next();
	    Chain attrType = pFV->valAsChain();
	    
	    if ( attrType == Chain("key") )
	    {
		keyTable=tableName;
		keyAttrList.Insert(attrName);
	    }
	    else if ( attrType == Chain("reference") )
	    {
		refTable=tableName;
		refAttrList.Insert(attrName);		
	    }
	    
	    fvl.Empty();	
	}

	Chain cmd;	

	cmd=Chain("ALTER TABLE DROP FOREIGN KEY") + *pKey + Chain(";\n");
	_pDF->writeChain(cmd);

	cmd=Chain("ALTER TABLE ") + keyTable + Chain(" ADD FOERIEGN KEY ") + *pKey + Chain("(");
	Chain *pKeyAttr = keyAttrList.First();
	while ( pKeyAttr )
	{
	    cmd += *pKeyAttr;
	    pKeyAttr = keyAttrList.Next();
	    if ( pKeyAttr )
		cmd += Chain(",");
	}

	cmd+=Chain(") REFERENCES ") + refTable + Chain("(");
	Chain *pRefAttr = refAttrList.First();
	while ( pRefAttr )
	{
	    cmd += *pRefAttr;
	    pRefAttr = refAttrList.Next();
	    if ( pRefAttr )
		cmd += Chain(",");
	}

	cmd += Chain(");\n");
	_pDF->writeChain(cmd);

	cout << " ... ok" << endl;
	pKey = keyList.Next();
    }
    
	
    res = _pCegoNet->doQuery(Chain("list check;"));
    
    schema.Empty();
    fvl.Empty();
    
    _pCegoNet->getSchema(schema);
	
    ListT<Chain> checkList;
    while ( _pCegoNet->fetchData(schema, fvl)  )
    {
	CegoFieldValue *pFV = fvl.First();
	Chain checkName = pFV->valAsChain();
	checkList.Insert(checkName);
	fvl.Empty();	
    }
    
    Chain *pCheck = checkList.First();
    while ( pCheck ) 
    {
	res = _pCegoNet->doQuery(Chain("desc check ") + *pCheck + Chain(";"));
	
	ListT<CegoField> schema;
	_pCegoNet->getSchema(schema);
	
	ListT<CegoFieldValue> fvl;
	bool isFirst=true;
	while ( _pCegoNet->fetchData(schema, fvl) );

	CegoFieldValue *pFV = fvl.First();
	Chain checkTable = pFV->valAsChain();
	pFV = fvl.Next();
	Chain checkCondition = pFV->valAsChain();
	
	fvl.Empty();	
       
	Chain cmd;	

	cmd=Chain("DROP CONSTRAINT IF EXISTS ") + *pCheck + Chain(";\n");
	_pDF->writeChain(cmd);

	cmd=Chain("ALTER TABLE ") +  checkTable + Chain(" add check ") + *pCheck + Chain(" on ") + checkCondition + Chain(";\n");
	_pDF->writeChain(cmd);

	pCheck = checkList.Next();
    }
}

Chain MySQLFormater::formatProcedure(const Chain& procName, CegoProcedure::ProcType procType, CegoDataType returnType, int returnTypeLen, int returnTypeDim, CegoProcBlock* pBlock)
{


    Chain s;

    if ( procType == CegoProcedure::FUNCTION )
	s = Chain("CREATE FUNCTION ");
    else
	s = Chain("CREATE PROCEDURE ");
    
    s += procName + Chain("(");
    CegoProcVar *pVar = pBlock->getVarList().First();
    while ( pVar )
    {
	if ( pVar->getVarType() == CegoProcVar::BLOCKVAR )
	    pVar = 0;
	else
	{
	    s += pVar->dbFormat(this);
	    pVar = pBlock->getVarList().Next();
	    	    
	    if ( pVar )
	    {
		if ( pVar->getVarType() != CegoProcVar::BLOCKVAR )
		s += Chain(",\n     ");
	    }
	}
    } 

    s += Chain(")");
    if ( procType == CegoProcedure::FUNCTION )
    {
	s += Chain(" RETURNS");
	switch ( returnType )
	{
	case INT_TYPE:
	    s += Chain(" INT");
	    break;
	case LONG_TYPE:
	    s += Chain(" LONG");
	    break;
	case VARCHAR_TYPE:
	    s += Chain(" VARCHAR(");
	    s += Chain(returnTypeLen);
	    s += Chain(")");
	    break;
	case BOOL_TYPE:
	    s += Chain(" BOOL");
	    break;
	case DATETIME_TYPE:
	    s += Chain(" DATETIME");
	    break;
	case BIGINT_TYPE:
	    s += Chain(" bigint(");
	    s += Chain(returnTypeLen);
	    s += Chain(")");
	    break;
	case FLOAT_TYPE:
	    s += Chain(" float");
	    break;
	case DOUBLE_TYPE:
	    s += Chain(" double");
	    break;
	case DECIMAL_TYPE:
	    s += Chain(" decimal(");
	    s += Chain(returnTypeLen);
	    s += Chain(")");
	    break;
	case FIXED_TYPE:
	    s += Chain(" fixed(");
	    s += Chain(returnTypeLen);
	    s += Chain(")");
	    break;
	case SMALLINT_TYPE:
	    s += Chain(" smallint");
	    break;
	case TINYINT_TYPE:
	    s += Chain(" tinyint");
	    break;	  
	case NULL_TYPE:
	    s += Chain(" null");
	    break;
	case BLOB_TYPE:
	case CLOB_TYPE:
	case PAGEID_TYPE:
	    throw Exception(EXLOC, Chain("Type not supported as return type"));
	}
    }

    s += Chain(" DETERMINISTIC \n");
    s += Chain("BEGIN\n");

    _indentCount = 1;

    s += formatIndent() + Chain("DECLARE NO_MORE_ROWS BOOLEAN;\n");


    pVar = pBlock->getVarList().First();
    while ( pVar )
    {
	if ( pVar->getVarType() == CegoProcVar::BLOCKVAR )
	   s += formatIndent() + Chain("DECLARE ") + pVar->dbFormat(this) + Chain(";\n");
	pVar = pBlock->getVarList().Next();
    }
    
    CegoProcStmt** pStmt = pBlock->getStmtList().First();
    while ( pStmt )
    {
	(*pStmt)->dbFormat(this);
	pStmt = pBlock->getStmtList().Next();
    }

    Chain *pDecStmt = _declareList.First();
    while ( pDecStmt )
    {
	s += *pDecStmt;
	pDecStmt = _declareList.Next();
    }

    s += formatIndent() + Chain("DECLARE CONTINUE HANDLER FOR NOT FOUND SET NO_MORE_ROWS = TRUE;\n");
    
    _declareList.Empty();

    s += pBlock->dbFormat(this);
    
    s += Chain("END\n");
    return s;

}

Chain MySQLFormater::formatProcVar(const Chain& varName, CegoProcVar::VarType varType, CegoDataType dt, int dtlen, int dtdim)
{
    
    Chain s = Chain("l_") + varName;
    
    switch ( dt )
    {
    case INT_TYPE:
	s += Chain(" INT");
	break;
    case LONG_TYPE:
	s += Chain(" BIGINT");
	break;
    case VARCHAR_TYPE:
	s += Chain(" VARCHAR(");
	s += Chain(dtlen);
	s += Chain(")");
	break;
    case BOOL_TYPE:
	s += Chain(" BOOL");
	break;
    case DATETIME_TYPE:
	s += Chain(" DATETIME");
	break;
    case BIGINT_TYPE:
	s += Chain(" DECIMAL(");
	s += Chain(dtlen);
	s += Chain(")");
	break;
    case FLOAT_TYPE:
	s += Chain(" FLOAT");
	break;
    case DOUBLE_TYPE:
	s += Chain(" DOUBLE");
	break;
    case DECIMAL_TYPE:
	s += Chain(" decimal(");
	s += Chain(dtlen);
	s += Chain(")");
	break;
    case FIXED_TYPE:
	s += Chain(" DECIMAL(");
	s += Chain(dtlen);
	s += Chain(")");
	break;
    case SMALLINT_TYPE:
	s += Chain(" SMALLINT");
	break;
    case TINYINT_TYPE:
	s += Chain(" TINYINT");
	break;	  
    case NULL_TYPE:
	s += Chain(" null");
	break;
    case BLOB_TYPE:
    case CLOB_TYPE:
    case PAGEID_TYPE:
	throw Exception(EXLOC, Chain("Type not supported as return type"));
    }

    return s;
    
}

Chain MySQLFormater::formatProcBlock(const ListT<CegoProcVar>& varList, const ListT<CegoProcStmt*>& stmtList,  const ListT<CegoProcException*>& exceptionList)
{
    
    Chain s;
    
    CegoProcStmt **pStmt = stmtList.First();
    while ( pStmt )
    {
	s += formatIndent() + (*pStmt)->dbFormat(this) + Chain("\n");
	pStmt = stmtList.Next();
    }
    
    CegoProcException **pException = exceptionList.First();
    while ( pException ) 
    {
	s += formatIndent() + Chain("EXCEPTION WHEN ") + (*pException)->dbFormat(this) + Chain(" THEN \n");	
	s += formatIndent() + (*pException)->getBlock()->dbFormat(this);
	pException = exceptionList.Next();
    }

    
    return s;
}


Chain MySQLFormater::formatProcThrowStmt(CegoExpr* pExpr)
{
    Chain s;    
    s = Chain("throw ") + pExpr->dbFormat(this) + Chain(";");
    return s;
}

Chain MySQLFormater::formatProcAssignStmt(CegoProcAssignStmt::AssignmentMode mode, const Chain paramName, CegoExpr* pExpr)
{
    Chain s;

    switch ( mode )
    {
    case CegoProcAssignStmt::EXPR2NULL:
    {
	_isPlain = true;
	s = pExpr->dbFormat(this) + Chain(";");
	_isPlain = false;
	return s;
    }
    case CegoProcAssignStmt::EXPR2PARAM:
    {
	s = Chain("SET l_") + paramName + " = " + pExpr->dbFormat(this) + Chain(";");
    }
    }

    return s;
}

Chain MySQLFormater::formatProcBlockStmt(CegoProcBlock* pBlock)
{
    Chain s;

    s = Chain("BEGIN\n");
    _indentCount++;
    
    s += pBlock->dbFormat(this);
    
    s += formatIndent() + Chain("END;");

    return s;


}

Chain MySQLFormater::formatProcCursorCreateStmt(const Chain& name, CegoSelect* pSelect)
{
    Chain s = Chain("DECLARE ") + name + Chain(" CURSOR FOR ") + pSelect->dbFormat(this) + Chain(";\n");

    _declareList.Insert(s);
    return Chain("DO 0;");

}

Chain MySQLFormater::formatProcCursorCloseStmt(const Chain& name)
{
    Chain s = Chain("CLOSE ") + name + Chain(";");
    return s;
}

Chain MySQLFormater::formatProcIfStmt(const ListT<CegoProcCond*>& condList, const ListT<CegoProcBlock*>& ifBlockList)
{

    Chain s;

    CegoProcCond** pCond = condList.First();
    CegoProcBlock** pBlock = ifBlockList.First();
    
    s = Chain("IF ");
    
    while ( pCond )
    {
	
	s += (*pCond)->dbFormat(this);
	
	s += Chain("\n") + formatIndent() + Chain("THEN\n");

	_indentCount++;
	s += (*pBlock)->dbFormat(this);
	_indentCount--;
	
	pCond = condList.Next();
	pBlock = ifBlockList.Next();
	if ( pCond )
	    s += formatIndent() + Chain("ELSEIF ");
	else if ( pBlock == 0 )
	    s += formatIndent() + Chain("END IF;");
    }
    if ( pBlock )
    {
	s += formatIndent() + Chain("ELSE\n");
	_indentCount++;
	s += (*pBlock)->dbFormat(this);
	_indentCount++;
	s += formatIndent() + Chain("END IF;");
    }

    return s;
}

Chain MySQLFormater::formatProcNoopStmt()
{
    Chain s = formatIndent();
    s += Chain("DO 0;");
    return s;
}

Chain MySQLFormater::formatProcQueryStmt(CegoQuery *pQuery)
{
    Chain s = pQuery->dbFormat(this) + Chain(";");

    Chain *pStmt = _setList.First();
    if ( pStmt )
	s += Chain("\n");
    
    while ( pStmt )
    {
	s += *pStmt + Chain(";\n");
	pStmt = _setList.Next();
    }
    _setList.Empty();
    
    return s;
}

Chain MySQLFormater::formatQuery(CegoQuery::QueryMode mode, const Chain& tableName, const Chain& tableAlias, const ListT<CegoField>& schema, const ListT<ListT<CegoExpr*> >& exprListArray,
				 const ListT<CegoExpr*>& updList, const ListT<CegoReturnVar*>& retVarList, const ListT<CegoAlterDesc>& alterList, CegoPredDesc *pPred, CegoSelect *pSelect,
				 const Chain& objName, CegoObject::ObjectType objType, const Chain& newObjName)
{
    Chain s;
    
    switch ( mode )
    {
    case CegoQuery::START_QUERY:
    {
	s = Chain("START TRANSACTION");
	break;
    }
    case CegoQuery::COMMIT_QUERY:
    {
	s = Chain("COMMIT");
	break;
    }
    case CegoQuery::ROLLBACK_QUERY:
    {
	s = Chain("ROLLBACK");
	break;
    }
    case CegoQuery::INSERT_QUERY:
    {

	
	s = Chain("INSERT INTO ") + tableName;

	if ( schema.Size() > 0 )
	{
	    CegoField* pFV = schema.First();
	    
	    s += " ( ";	
	    
	    while ( pFV )
	    {
		s += pFV->getAttrName();
		pFV = schema.Next();
		if ( pFV  )
		    s += Chain(", ");
	    }
	    
	    s += Chain(" )"); 
	}
	s += Chain("\n") + Chain(" values ");

	ListT<CegoExpr*> *pExprList = exprListArray.First();
	while ( pExprList )
	{			    

	    s += Chain("("); 
	
	    CegoExpr **pExpr = pExprList->First();
	    while ( pExpr )
	    {
		s += (*pExpr)->dbFormat(this);
		pExpr = pExprList->Next();
		if ( pExpr )
		    s += Chain(", ");
	    }
	    s += Chain(")");
	    pExprList = exprListArray.Next();
	    if ( pExprList )
		s += Chain(", ");		
	}
	break;
    }
    case CegoQuery::INSERTBYSELECT_QUERY:
    {

	s =  Chain("INSERT INTO ") + tableName;

	if ( schema.Size() > 0 )
	{
	    CegoField* pFV = schema.First();
	    
	    s += Chain(" ( ");	
	    
	    while ( pFV )
	    {
		s += pFV->getAttrName();
		pFV = schema.Next();
		if ( pFV  )
		    s += Chain(", ");
	    }
	    
	    s += Chain(" )"); 
	}
	s += pSelect->dbFormat(this);

	break;
    }
    case CegoQuery::DELETE_QUERY:
    {
	s = Chain("DELETE FROM ") + tableName;

	if ( tableName != tableAlias )
	    s += Chain(" ") + tableAlias;

	if (pPred)
	{
	    s += Chain(" WHERE ");
	    s += pPred->dbFormat(this);
	}
	break;
    }
    case CegoQuery::LOCK_QUERY:
    {
	// s = Chain("lock table xxx");
	break;
    }
    case CegoQuery::UNLOCK_QUERY:
    {
	// s = Chain("unlock table xxx");
	break;
    }
    case CegoQuery::UPDATE_QUERY:
    {

	s = Chain("UPDATE ") + tableName;

	if ( tableName != tableAlias )
	    s += Chain(" ") + tableAlias;
	
	s += Chain(" SET ");

	CegoField* pFV = schema.First();
	CegoExpr** pExpr = updList.First();
		    
	while ( pFV && pExpr)
	{
	    (*pExpr)->dbFormat(this);
	    
	    s += pFV->getAttrName() + Chain("=") + (*pExpr)->dbFormat(this);
	    pFV = schema.Next();
	    pExpr = updList.Next();
	    if ( pFV && pExpr )
		s += Chain(", ");
	}
	
	if ( pPred )
	{
	    s += Chain(" WHERE ");
	    s += pPred->dbFormat(this);
	}


	if ( retVarList.isEmpty() == false )
	{ 

	    CegoReturnVar **pRV = retVarList.First();
	    while ( pRV ) 
	    {
		Chain setStmt = Chain ("SET l_") + (*pRV)->getVarName();
		CegoExpr* pExpr = (*pRV)->getExpr();

		setStmt += Chain("= ( SELECT ") + pExpr->dbFormat(this) + Chain(" FROM ") + tableName;

		if ( tableName != tableAlias )
		    setStmt += Chain(" ") + tableAlias;

		if ( pPred )
		{
		    setStmt += Chain(" WHERE ") + pPred->dbFormat(this);
		}

		setStmt += Chain(")");

		_setList.Insert(setStmt);
		
		pRV = retVarList.Next();

	    }
	} 
	

	break;
    }
    case CegoQuery::ALTER_QUERY:
    {
	s = Chain("ALTER ") + tableName + Chain(" ");
	CegoAlterDesc *pAD = alterList.First();
	while ( pAD ) 
	{
	    s += pAD->dbFormat(this);
	    pAD = alterList.Next();
	    if ( pAD ) 
		s += ",";
	}
	break;
    }
    case CegoQuery::RENAME_QUERY:
    {
	s = Chain("RENAME ");
	switch ( objType )
	{
	case CegoObject::TABLE:
	{
	    s += Chain("TABLE ");
	    break;
	}
	case CegoObject::AVLTREE:
	case CegoObject::PAVLTREE:
	case CegoObject::UAVLTREE:
	case CegoObject::BTREE:
	case CegoObject::PBTREE:
	case CegoObject::UBTREE:
	{
	    s += Chain("INDEX ");
	    break;
	}
	case CegoObject::FKEY:
	{
	    s += Chain("KEY ");
	    break;
	}
	case CegoObject::PROCEDURE:
	{
	    s += Chain("PROCEDURE ");
	    break;
	}
	case CegoObject::VIEW:
	{
	    s += Chain("VIEW ");
	    break;
	}
	default:
	    break;
	}
	s += objName + Chain(" TO ") + newObjName;

	break;
    }
    }

    return s;
    
}

Chain MySQLFormater::formatProcReturnStmt(CegoExpr *pExpr)
{
    Chain s = Chain("RETURN ") + pExpr->dbFormat(this) + Chain(";");
    return s;
}


Chain MySQLFormater::formatProcWhileStmt(CegoProcCond* pCond, CegoProcBlock* pBlock)
{

    Chain s;
    Chain fetch = Chain("");
    Chain cond = pCond->dbFormat(this);
    
    if ( _fetchStmt != Chain(""))
    {
	fetch = _fetchStmt;
	_fetchStmt = Chain("");
    }

    if ( fetch != Chain("") )
	s += formatIndent() + fetch + Chain(";\n");
    
    s += Chain("WHILE ") + cond + Chain(" DO\n");
    _indentCount++;
    
    s += pBlock->dbFormat(this);
    
    if ( fetch != Chain(""))
	s += formatIndent() + fetch + Chain(";\n");
    
    _indentCount--;
    s += Chain("END WHILE;");

    return s;

}

Chain MySQLFormater::formatProcCond(CegoProcCond::ConditionMode mode, CegoProcCond* pCond, CegoProcPred* pPred)
{
    Chain s;

    switch ( mode )
    {
    case CegoProcCond::AND:
    {
	s = pCond->dbFormat(this) + " AND " + pPred->dbFormat(this);
	break;
    }
    case CegoProcCond::OR:
    {
	s = pCond->dbFormat(this) + " OR " + pPred->dbFormat(this);
	break;
    }
    case CegoProcCond::PRED:
    {
	s = pPred->dbFormat(this);
    }
    }
    
    return s;
}

Chain MySQLFormater::formatProcPred(CegoProcPred::PredicateMode mode, CegoExpr* pExpr1, CegoExpr* pExpr2, CegoComparison comp, CegoProcCond* pCond, bool isNegate)
{
    Chain s;
    if ( mode == CegoProcPred::CONDITION )
    {
	s = pCond->dbFormat(this);
    }
    else if ( mode == CegoProcPred::EXPRCOMP )
    {
	s = pExpr1->dbFormat(this);

	switch (comp)
	{
	case EQUAL:
	{
	    s += " = ";
	    break;
	}
	
	case LESS_THAN:
	{
	    s += " < ";
	    break;
	}	
	case MORE_THAN:
	{
	    s += " > ";
	    break;
	}
	case LESS_EQUAL_THAN:
	{
	    s += " <= ";
	    break;	    
	}
	case MORE_EQUAL_THAN:
	{
	    s += " >= ";
	    break;	    
	}
	case NOT_EQUAL:
	{
	    s += " != ";
	    break;	    
	}
	}

	s += pExpr2->dbFormat(this);
    }
    else if ( mode == CegoProcPred::NULLCOMP )
    {
	s = pExpr1->dbFormat(this);
	if ( isNegate )
	{
	    s += " IS NOT NULL";
	}
	else
	{
	    s += " IS NULL";
	}
    }
    return s;
}

Chain MySQLFormater::formatProcException(CegoException excep)
{
    return Chain("EXCEP TODO");
}

Chain MySQLFormater::formatExpr(CegoExpr::ExpType expType, CegoExpr* pExpr, CegoTerm* pTerm)
{
    Chain s;
    switch ( expType )
    {
    case CegoExpr::ADD:
	s =  pExpr->dbFormat(this) + " + " + pTerm->dbFormat(this);
	break;
    case CegoExpr::SUB:
	s = pExpr->dbFormat(this) + " - " + pTerm->dbFormat(this);
	break;
    case CegoExpr::CONCAT:
	s = pExpr->dbFormat(this) + " | " + pTerm->dbFormat(this);
	break;
    case CegoExpr::TERM:
	s = pTerm->dbFormat(this);
	break;
    }
    return s;

}

Chain MySQLFormater::formatTerm(CegoTerm::TermType termType, CegoTerm* pTerm, CegoFactor* pFactor)
{
    Chain s;
    switch ( termType )
    {
    case CegoTerm::MUL:
	s = pTerm->dbFormat(this) + " * " + pFactor->dbFormat(this);
	break;
    case CegoTerm::DIV:
	s = pTerm->dbFormat(this) + " / " + pFactor->dbFormat(this);
	break;
    case CegoTerm::FACTOR:
	s = pFactor->dbFormat(this);
	break;
    }
    return s;    
}

Chain MySQLFormater::formatFactor(CegoFactor::FacType facType, const CegoFieldValue& fv, CegoAttrDesc* pAttrDesc, const Chain varName,
				  CegoExpr* pExpr, CegoProcFetch *pFetch, CegoAggregation* pAggr, CegoFunction* pFunction,
				  CegoSelect* pSelect, CegoCaseCond* pCaseCond )
{

    Chain s;
    switch (facType)
    {
    case CegoFactor::CONSTVAL:
	s = fv.toChain();
	break;
    case CegoFactor::ATTR:
	s = pAttrDesc->toChain();
	break;
    case CegoFactor::VAR:
	s = Chain("l_") + varName;
	break;
    case CegoFactor::EXPR:
	s = Chain("(") + pExpr->dbFormat(this) + Chain(")");
	break;
    case CegoFactor::FETCH:
	s = pFetch->dbFormat(this);
	break;
    case CegoFactor::AGGREGATION:
	s = pAggr->dbFormat(this);
	break;
    case CegoFactor::FUNCTION:
	if ( _isPlain )
	    s = Chain("CALL ") + pFunction->dbFormat(this);
	else
	    s = + pFunction->dbFormat(this);
	break;
    case CegoFactor::QUERY:
	s = Chain("(") + pSelect->dbFormat(this) + Chain(")"); 
	break;
    case CegoFactor::CASECOND:
	s = pCaseCond->dbFormat(this);
	break;
    }
    return s;

}

    
Chain MySQLFormater::formatSelect(CegoSelect::SelectMode mode, bool isDistinct, const ListT<CegoExpr*>& exprList,
				  const ListT<CegoContentObject*>& coList, CegoPredDesc *pPred, ListT<CegoAttrDesc*> *pGroupList, CegoHavingDesc* pHaving, ListT<CegoExpr*> *pOrderList, int rowLimit, CegoSelect* pUnionSelect)
{

    Chain s;

    s = Chain("SELECT");

    if ( isDistinct && mode != CegoSelect::AGGREGATION )
	s += Chain(" DISTINCT");
    
    if ( exprList.Size() == 0 )
    {
	s += Chain(" *\n");
    }
    else
    {
	s += Chain("\n");
	CegoExpr** pExpr = exprList.First();
	while (pExpr)
	{
	    s += (*pExpr)->dbFormat(this);
	    if ( (*pExpr)->getAlias() != Chain() )
	    {
		s += Chain(" as ") + (*pExpr)->getAlias();
	    } 
	    pExpr = exprList.Next();
	    if ( pExpr )
		s += Chain(",\n");
	}
    }

    s += Chain("\n") + formatIndent() + Chain("FROM\n");

    CegoContentObject** pCO = coList.First();
    while (pCO)
    {
	s += formatIndent() + Chain(" ") + (*pCO)->getTabName() + Chain(" ") + (*pCO)->getTabAlias() ;;
	pCO = coList.Next();
	if ( pCO )
	    s += Chain(",\n");
    }
    

    if (pPred)
    {
	s += Chain("\n");
	s += formatIndent() + Chain("WHERE\n");
	s += formatIndent() + pPred->dbFormat(this);
	s += Chain("\n");
    }

    if (pGroupList)
    {
	s += Chain("\n");
	s += formatIndent() + Chain("GROUP BY\n");
	
	CegoAttrDesc **pAttrDesc = pGroupList->First();
	while ( pAttrDesc )
	{
	    s += formatIndent() + (*pAttrDesc)->toChain();
	    pAttrDesc = pGroupList->Next();
	    if ( pAttrDesc )
		s += ",";
	}
	
	if ( pHaving )
	{
	    s += Chain("\n") + formatIndent() + Chain("HAVING\n");
	    s += formatIndent() + pHaving->dbFormat(this);
	}
    }
    if ( pOrderList )
    {
	s += Chain("\n");
	s += formatIndent() + Chain("ORDER BY\n");
	
	CegoExpr **pExpr = pOrderList->First();
	while ( pExpr )
	{
	    s += (*pExpr)->dbFormat(this);
	    pExpr = pOrderList->Next();
	    if ( pExpr )
		s += ",";
	}
    }

    if ( pUnionSelect )
    {
	s += Chain("\n");
	s += formatIndent() + Chain("UNION ALL\n");
	s += pUnionSelect->dbFormat(this);
    }
    return s;

}

Chain MySQLFormater::formatProcFetch(const Chain& curName, const ListT<Chain>& varList)
{

    _fetchStmt = Chain("FETCH ") + curName + Chain(" INTO ");
    
    Chain *pVar = varList.First();
    while ( pVar )
    {
	_fetchStmt += Chain("l_") + *pVar;
	pVar = varList.Next();
	if ( pVar )
	    _fetchStmt += Chain(", ");
    }

    if ( _isPlain )
	return _fetchStmt;
    
    return Chain("NO_MORE_ROWS");

}

Chain MySQLFormater::formatHaving(CegoExpr *pAggExpr, CegoExpr* pExpr, CegoComparison comp)
{
    Chain s;

    s += pAggExpr->dbFormat(this);
    
    switch ( comp )
    {
    case EQUAL:
	s += " = ";
	break;
    case NOT_EQUAL:
	s += " != ";
	break;
    case LESS_THAN:
	s += " < ";
	break;
    case MORE_THAN:
	s += " > ";
	break;
    case LESS_EQUAL_THAN:
	s += " <= ";
	break;
    case MORE_EQUAL_THAN:
	s += " >= ";
	break;
    }
    
    s += pExpr->dbFormat(this);
    
    return s;

}
    
Chain MySQLFormater::formatPred(CegoPredDesc::CompMode mode, CegoExpr* pExpr1, CegoExpr *pExpr2, CegoExpr* pExpr3, ListT<CegoExpr*>& exprList, CegoComparison comp, const Chain& pattern, CegoCondDesc* pCond, CegoPredDesc* pPred, CegoSelect *pSelect)
{

    Chain s;

    switch ( mode )
    {
    case CegoPredDesc::CONDITION:
    {

	if ( pCond->Left())
	{
	    s = Chain("( ");
	    s += pCond->Left()->dbFormat(this);
	    s += Chain("\n");

	    switch( pCond->getCondType())
	    {
	    case CegoCondDesc::AND:
		s += Chain(" AND") + Chain("\n");
		break;
	    case CegoCondDesc::OR:
		s += Chain(" OR") + Chain("\n");
		break;
	    default:
		break;
	    }
	    s += pCond->Right()->dbFormat(this);
	    s += Chain(" )");
	}
	break;
    }
    case CegoPredDesc::NOTPRED:
    {
	s = formatIndent() + Chain("NOT\n") + pPred->dbFormat(this);
	break;
    }
    case CegoPredDesc::EXPRCOMP:
    {
	
	s = formatIndent() + pExpr1->dbFormat(this);
	
	switch ( comp )
	{
	case EQUAL:
	    s += " = ";
	    break;
	case NOT_EQUAL:
	    s += " != ";
	    break;
	case LESS_THAN:
	    s += " < ";
	    break;
	case MORE_THAN:
	    s += " > ";
	    break;
	case LESS_EQUAL_THAN:
	    s += " <= ";
	    break;
	case MORE_EQUAL_THAN:
	    s += " >= ";
	    break;
	}
	s +=  pExpr2->dbFormat(this);
	break;
    }
    case CegoPredDesc::BETWEEN:
    {
	s =  pExpr2->dbFormat(this);
	s += Chain(" BETWEEN ");
	s +=  pExpr1->dbFormat(this);
	s += Chain(" AND ");
	s +=  pExpr3->dbFormat(this);
	break;
    }
    case CegoPredDesc::ISLIKE:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" LIKE ");
	s += Chain("'") + pattern + Chain("'");
	break;
    }
    case CegoPredDesc::ISNOTLIKE:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" NOT LIKE ");
	s += Chain("'") + pattern + Chain("'");
	break;
    }
    case CegoPredDesc::NULLCOMP:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" IS NULL ");
	break;
    }
    case CegoPredDesc::NOTNULLCOMP:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" IS NOT NULL ");
	break;
    }
    case CegoPredDesc::EXISTSCOMP:
    {
	s = Chain("EXISTS (\n");
	s += pSelect->dbFormat(this) + ")";
	break;
    }
    case CegoPredDesc::IN:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" IN ");
	s += Chain("(");

	CegoExpr** pExpr = exprList.First();
	while ( pExpr )
	{
	    s += (*pExpr)->dbFormat(this);
	    pExpr = exprList.First();
	    if ( pExpr )
		s += Chain(",");
	}
	s += Chain(")");   
	break;
    }
    case CegoPredDesc::NOTIN:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" NOT IN ");
	s += Chain("(");

	CegoExpr** pExpr = exprList.First();
	while ( pExpr )
	{
	    s += (*pExpr)->dbFormat(this);
	    pExpr = exprList.First();
	    if ( pExpr )
		s += Chain(",");
	}
	s += Chain(")");   
	break;
    }    
    case CegoPredDesc::INSUB:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" IN ");
	s += Chain("(") + pSelect->dbFormat(this) + Chain(")");   
	break;
    }
    case CegoPredDesc::NOTINSUB:
    {
	s = pExpr1->dbFormat(this);
	s += Chain(" NOT IN ");
	s += Chain("(") + pSelect->dbFormat(this) + Chain(")");   
	break;
    }  
    }
    return s;
}
    
Chain MySQLFormater::formatAttr(const Chain& tableName, const Chain& attrName)
{
    return tableName + Chain(".") + attrName;
}
    
Chain MySQLFormater::formatFunction(CegoFunction::FunctionType type, const ListT<CegoExpr*>& exprList, const Chain& funcName, const Chain& counterId)
{
    Chain argString;

    CegoExpr** pExpr = exprList.First();
    while ( pExpr )
    {
	argString += (*pExpr)->dbFormat(this);   
	pExpr = exprList.Next();
	if ( pExpr )
	    argString += Chain(",");
    }

    Chain funcString;

    switch (type)
    {

    case CegoFunction::TRIM:
    {
	funcString = Chain("TRIM");
	break;	
    }
    case CegoFunction::RTRIM:
    {
	funcString = Chain("RTRIM");
	break;	
    }
    case CegoFunction::LTRIM:
    {
	funcString = Chain("LTRIM");
	break;	
    }
    case CegoFunction::ROUND:
    {
	funcString = Chain("ROUND");
	break;	
    }
    case CegoFunction::TRUNC:
    {
	funcString = Chain("TRUNC");
	break;	
    }
    case CegoFunction::NEWDATE:
    {
	funcString = Chain("NOW");
	break;	
    }
    case CegoFunction::DATE2STR:
    {
	funcString = Chain("DATE_FORMAT");
	break;	
    }
    case CegoFunction::DATE2LONG:
    {
	funcString = Chain("UNIX_TIMESTAMP");
	break;	
    }
    case CegoFunction::LONG2DATE:
    {
	funcString = Chain("FROM_UNIXTIME");
	break;	
    }
    case CegoFunction::LOWER:
    {
	funcString = Chain("LOWER");
	break;	
    }
    case CegoFunction::UPPER:
    {
	funcString = Chain("UPPER");
	break;	
    }
    case CegoFunction::LEFT:
    {
	funcString = Chain("LEFT");
	break;	
    }
    case CegoFunction::RIGHT:
    {
	funcString = Chain("RIGHT");
	break;	
    }
    case CegoFunction::SUBSTR:
    {
	funcString = Chain("SUBSTR");
	break;	
    }
    case CegoFunction::GETPOS:
    {
	funcString = Chain("GETPOS");
	break;		
    }
    case CegoFunction::STR2INT:
    {
	Chain s = Chain("CONVERT (") + argString + Chain(" ,SIGNED)");
	return s;	
    }
    case CegoFunction::INT2ASC:
    {
	Chain s = Chain("CONVERT (") + argString + Chain(" , CHAR CHARACTER SET utf8)");
	return s;	
    }
    case CegoFunction::ASC2INT:
    {
	Chain s = Chain("CONVERT (") + argString + Chain(" , DECIMAL)");
	return s;	
    }
    case CegoFunction::LMOD:
    {
	funcString = Chain("MOD");
	break;
    }
    case CegoFunction::LDIV:
    {
	funcString = Chain("DIV");
	break;
    }
    case CegoFunction::STR2LONG:
    {
	Chain s = Chain("CONVERT (") + argString + Chain(" ,DECIMAL)");
	return s;	
    }
    case CegoFunction::STR2DATE:
    {
	Chain s = funcString + Chain("CONVERT (DATE_FORMAT(") + argString + Chain(") ,DATETIME)");
	return s;	
    }
    case CegoFunction::RANDSTR:
    {
	Chain s = funcString + Chain("LEFT(MD5(RAND()),") + argString + Chain(")");
	return s;
    }
    case CegoFunction::RANDINT:
    {
	Chain s = funcString + Chain("ROUND(RAND() * ") + argString + Chain(")");
	return s;
    }
    case CegoFunction::MOD:
    {
	funcString = Chain("MOD");
	break;		
    }
    case CegoFunction::DIV:
    {
	funcString = Chain("DIV");
	break;		
    }
    case CegoFunction::POWER:
    {
	funcString = Chain("POWER");
	break;		
    }
    case CegoFunction::BITAND:
    {
	funcString = Chain("BITAND");
	break;		
    }
    case CegoFunction::BITOR:
    {
	funcString = Chain("BITOR");
	break;		
    }
    case CegoFunction::BITXOR:
    {
	funcString = Chain("BITXOR");
	break;		
    }
    case CegoFunction::BLOBSIZE:
    {
	funcString = Chain("BLOBSIZE");
	break;		
    }
    case CegoFunction::CLOBSIZE:
    {
	funcString = Chain("CLOBSIZE");
	break;		
    }
    case CegoFunction::BLOBREF:
    {
	funcString = Chain("BLOBREF");
	break;		
    }
    case CegoFunction::CLOBREF:
    {
	funcString = Chain("CLOBREF");
	break;		
    }
    case CegoFunction::CLOB2STR:
    {
	funcString = Chain("CLOB2STR");
	break;		
    }
    case CegoFunction::REPLACE:
    {
	funcString = Chain("REPLACE");
	break;	
    }
    case CegoFunction::LENGTH:
    {
	funcString = Chain("LENGTH");
	break;	
    }
    case CegoFunction::NEXTCOUNT:
    {
	funcString = Chain("NEXTCOUNT");
	argString = Chain("'") + counterId + Chain("'");
	break;	
    }
    case CegoFunction::SETCOUNT:
    {
	funcString = Chain("SETCOUNT");
	argString = Chain("'") + counterId + Chain("'") + Chain(",") + argString;
	break;	
    }
    case CegoFunction::USERDEFINED:
    {
	funcString = funcName;
	break;	
    }
    }

    Chain s = funcString + Chain("(") + argString + Chain(")");
    
    return s;
    
}

Chain MySQLFormater::formatAggregation(CegoAggregation::AggType type, CegoExpr *pExpr, bool isDistinct)
{
    
    Chain s;
    switch ( type )
    {
    case CegoAggregation::MIN:
	s =  Chain("MIN(") + pExpr->toChain() + Chain(")");
	break;
    case CegoAggregation::MAX:
	s =  Chain("MAX(") + pExpr->toChain() + Chain(")");
	break;
    case CegoAggregation::AVG:
	s =  Chain("AVG(") + pExpr->toChain() + Chain(")");
	break;
    case CegoAggregation::SUM:
	s =  Chain("SUM(") + pExpr->toChain() + Chain(")");
	break;
    case CegoAggregation::COUNT:
    {       
	if ( pExpr )
	{ 
	    Chain distOpt;
	    if ( isDistinct )
		distOpt = Chain(" DISTINCT ");	    
	    s = Chain("count(") + distOpt + pExpr->toChain() + Chain(")");
	}
	else
	    s = Chain("COUNT(*)");
    }
    }

    return s;

}

Chain MySQLFormater::formatCaseCond(const ListT<CegoPredDesc*>& predList, const ListT<CegoExpr*>& exprList, CegoExpr* elseExpr)
{
    return Chain("CASE TODO");
}


Chain MySQLFormater::formatReturnValue(const Chain& varName, CegoExpr* pExpr)
{
    return varName + Chain(" = ") + pExpr->dbFormat(this);
}

Chain MySQLFormater::formatAlter(CegoAlterDesc::AlterType type, const CegoField& f, const Chain& attrName)
{
    return Chain("ALTER TODO");
}

Chain MySQLFormater::formatFieldValue(CegoDataType type, void *pV, int len )
{
    Chain s;

    if ( type == NULL_TYPE || type == BLOB_TYPE )
    {
	s = Chain("null");
    }
    else
    {
	if ( pV == 0 )
	{
	    s = Chain("null");
	    return s;
	}
	switch ( type )
	{
	case INT_TYPE:
	{
	    int i;
	    memcpy(&i, pV, sizeof(int));
	    s = Chain( i );
	    break;
	}   
	case LONG_TYPE:
	{
	    long l;
	    memcpy(&l, pV, sizeof(long));
	    s = Chain( l );
	    break;
	}   
	case VARCHAR_TYPE:	    
	{   
	    Chain val((char*)pV);

	    // escape backslashes
	    Chain bval;
	    val.replaceAll(Chain("\\"), Chain("\\\\"), bval);

	    // escape newlines
	    Chain nval;
	    bval.replaceAll(Chain("\n"), Chain("\\n"), nval);

	    Chain qval;
	    nval.replaceAll(Chain("'"), Chain("\\'"), qval);

	    s = Chain("'") + qval + Chain("'");
	    break;
	}   
	case BOOL_TYPE:
	{   
	    char b = *(char*)(pV);
	    if ( b == 1 )
		s = Chain("true");
	    else
		s = Chain("false");
	    break;
	}    
	case DATETIME_TYPE:
	{
	    int i;
	    memcpy(&i, pV, sizeof(int));
	    if ( i == 0 )
	    {
		s = Chain("now()");
	    }
	    else
	    {
		Datetime dt ( i );
		s = Chain("'") + dt.asChain(MYSQLDATEFORMAT) + Chain("'");
	    }
	    break;
	}
	case BIGINT_TYPE:
	{
	    BigInteger bi( Chain((char*)pV) );
	    s =  bi.toChain();
	    break;
	}
	case DECIMAL_TYPE:
	{
	    BigDecimal d( Chain((char*)pV) );
	    s = d.toChain();
	    break;	    
	}
	case FIXED_TYPE:
	{
	    BigDecimal d( Chain((char*)pV) );
	    s = d.toChain();
	    break;
	}
	case FLOAT_TYPE:
	{
	    float f;
	    memcpy(&f, pV, sizeof(float));
	    s = Chain( f );
	    break;
	}
	case DOUBLE_TYPE:
	{
	    double d;
	    memcpy(&d, pV, sizeof(double));
	    s = Chain( d );
	    break;
	}
	case SMALLINT_TYPE:
	{
	    short st;
	    memcpy(&st, pV, sizeof(short));
	    s = Chain( st );
	    break;
	}
	case TINYINT_TYPE:
	{
	    s = Chain( *(char*)(pV) );
	    break;
	}
	case BLOB_TYPE:
	case CLOB_TYPE:
	case PAGEID_TYPE:
	case NULL_TYPE:
	    // already handled
	    break;	    
	}
    }
    return s;

}

Chain MySQLFormater::formatTrigger(const Chain& triggerName,
				   bool isBefore,
				   bool isOnInsert,
				   bool isOnUpdate,
				   bool isOnDelete,
				   const Chain& tableName,				
				   CegoProcBlock* pBlock)
{

    Chain s;

    s = Chain("CREATE TRIGGER ");
    
    s += triggerName;
    

    s += Chain("BEGIN\n");

    _indentCount = 1;

    CegoProcVar *pVar = pBlock->getVarList().First();

    while ( pVar )
    {
	if ( pVar->getVarType() == CegoProcVar::BLOCKVAR )
	   s += formatIndent() + Chain("DECLARE ") + pVar->dbFormat(this) + Chain(";\n");
	pVar = pBlock->getVarList().Next();
    }
    
    CegoProcStmt** pStmt = pBlock->getStmtList().First();
    while ( pStmt )
    {
	(*pStmt)->dbFormat(this);
	pStmt = pBlock->getStmtList().Next();
    }

    Chain *pDecStmt = _declareList.First();
    while ( pDecStmt )
    {
	s += *pDecStmt;
	pDecStmt = _declareList.Next();
    }
    
    _declareList.Empty();

    s += pBlock->dbFormat(this);
    
    s += Chain("END\n");
    return s;

}

Chain MySQLFormater::formatIndent() const
{
    Chain s;
    for ( int i=0; i < _indentCount; i++)
	s += INDENT;
    return s;
}
