#ifndef _ORACLEACTION_H_INCLUDED_
#define _ORACLEACTION_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// OracleAction.h  
// ------------------
// Oracle semantic actions interface definition
//
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2015 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: OracleAction
// 
// Description:
//
//
///////////////////////////////////////////////////////////////////////////////

#include <lfcbase/StackT.h>
#include <lfcbase/ListT.h>
#include <lfcbase/File.h>

#include <cego/CegoNet.h>
#include "OracleParser.h"

#define ORACLE_MAXSTRINGLEN 500000

class OracleAction : public OracleParser {
    
public:
    
    OracleAction(const Chain& fileName, CegoNet *pCegoNet, bool isCaseSensitive = false);
    ~OracleAction();

    void cleanUp();
    bool moreData();
    void setFile(const Chain& fileName);
    char nextChar();
    void readChain();
    void backChar();

    void printTokenList();

    /////////////////////////////////////////////////
    /////////// start of semantic actions ///////////
    /////////////////////////////////////////////////

    void postAction();
    
    void execCreate();
    void execInsert();
    void setInsertTable();
    void execDrop();
    void dropOpt1();
    void dropOpt2();
    void storeAttrColumn();
    void storeTable();

    void storePrimaryKey();
    void storeUniqueKey();
    void storeKey();
    void storeKeyColumn();
    void storeNullOpt();
    void storeNotNullOpt();
    void storeDefaultOpt();
    void storeDefaultNullOpt();
    void insertConstValue();
    void insertPreAction();
    void insertPostAction();
    void storeIntType();
    void storeDecimalType();
    void storeUnsignedType();
    void storeVarcharType();
    void storeTimestampType();
    void storeDatetimeType();
    void storeMediumTextType();
    void storeEnumType();
    void storeIntValue();
    void storeNegIntValue();
    void storeVarcharValue();
    void storeNullValue();
    void storeDim();
    void storeDecimalDim();

private:

    class TypeInfo {
	
    public:

	TypeInfo()
	{
	}

	TypeInfo(const Chain& tableName, int pos)
	{
	    _tableName = tableName;
	    _pos = pos;
	}
	
	TypeInfo(const Chain& tableName, int pos, const Chain& type)
	{
	    _tableName = tableName;
	    _pos = pos;
	    _type = type;
	}

	const Chain& getTable()
	{
	    return _tableName;
	}
	
	int getPos() 
	{
	    return _pos;
	}
	
	const Chain& getType()
	{
	    return _type;
	}
	
	TypeInfo& operator = ( const TypeInfo& ti)
	{
	    _tableName = ti._tableName;
	    _pos = ti._pos;
	    _type = ti._type;
	    return *this;
	}
	
	bool operator == ( const TypeInfo& ti) const
	{
	    return ti._tableName == _tableName && ti._pos == _pos;
	}
	
    private:

	Chain _tableName;
	int _pos;
	Chain _type;
    };
    
    ListT<TypeInfo> _typeList;

    void putTypeInfo(TypeInfo ti);
    TypeInfo* getTypeInfo(const Chain& tableName, int pos);

    int _i;
    char *_pC;
    Chain _line;

    int _stringBufLen;
    char _stringBuf[ORACLE_MAXSTRINGLEN];

    File* _pFile;
    CegoNet* _pCegoNet;

    bool _isFirst;
    bool _isCaseSensitive;

    ListT<Chain> _colList;
    ListT<Chain> _keyColList;
    ListT<Chain> _valList;

    ListT<Chain> _postList;
    
    Chain _dataTypeString;
    Chain _nullOptString;
    Chain _defaultOptString; 
    Chain _table;
    Chain _constValue;
    int _dim;
    int _decDim;

};

#endif














