///////////////////////////////////////////////////////////////////////////////
//                                                         
// PostgresAction.cc  
// --------------                                                        
// Postgres semantic parser actions
//
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2015 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: PostgresAction
// 
// Description: 
//
//
///////////////////////////////////////////////////////////////////////////////


#include <ctype.h>
#include <stdlib.h>

#include "PostgresAction.h"

#define POSTGRES_MAXLINELEN 10000000
#define MAXCGSTRLEN 5000

PostgresAction::PostgresAction(const Chain& fileName, CegoNet *pCegoNet, bool isCaseSensitive)
{
    _pFile = new File(fileName);   
    _pFile->open(File::READ);
    _i = 0;
    _pC = 0;
    _pCegoNet = pCegoNet;
    _isFirst=true;
    _isCaseSensitive = isCaseSensitive;
}

PostgresAction::~PostgresAction()  
{
}

bool PostgresAction::moreData()
{
    while ( 1 )
    {
	if ( _pFile->readLine(_line, POSTGRES_MAXLINELEN) > 0 )
	{	    
	    if ( _line.length() >= 2 )
	    {		    
		if (_line.cutTrailing(" \t").subChain(1, 2) != Chain("--"))
		{
		    _pC = (char*)_line;
		    _i=0;	       
		    return true;
		}
	    }
	}
	else
	{
	    return false;
	}
    }
}

char PostgresAction::nextChar() 
{   
    
    if ( _isFirst )
    {
	if ( moreData() == false )
	    return 0;
	_isFirst=false;
    }

    if ( *_pC == '\'')
    {
	_pC++;
	readChain();
	return 0;
    }

    if ( *_pC == '/' && *(_pC+1) == '*' )
    {
	_pC++;
	_pC++;

	bool eoc=false;
	while ( ! eoc )
	{
	    while ( *_pC != '*' )
	    {
		_pC++;
		if ( *_pC == 0 ) 
		{
		    if ( moreData() == false )
			return 0; 
		}
	    }
	    if ( *_pC == '*' && *(_pC+1) == '/' )
	    {
		eoc=true;
		_pC++;
	    }	   
	    _pC++;	    
	}	
    }

    if ( *_pC == 0 ) 
    {
	if ( moreData() )
	    return nextChar();
	else
	    return 0; 
    }
        
    char c = *_pC; 
    _pC++; 

    if ( _isCaseSensitive )
	return c;
    else
	return (char)tolower((int)c);
} 

void PostgresAction::readChain()
{

    setReserved(STRINGVAL); 
    
    int i=0;    

    
    while ( *_pC != '\'') 
    { 
	// cout << "Storing: " << *_pC << endl; 
	
	if ( *_pC == '\\') 
	{ 
	    _pC++; 
	    
	    if ( *_pC == 'n') 
	    { 
		_stringBuf[i] = '\n'; 
	    } 
	    else if ( *_pC == 't') 
	    { 
		_stringBuf[i] = '\t'; 
	    } 
	    else if ( *_pC == 'r') 
	    { 
		_stringBuf[i] = '\r'; 
	    } 
	    else if ( *_pC == '0') 
	    { 
		_stringBuf[i] = '\0'; 
	    }
	    else if ( *_pC == '"') 
	    { 
		_stringBuf[i] = '"'; 
	    } 
	    else if ( *_pC == '\'') 
	    { 
		_stringBuf[i] = 'X'; 
	    }

	    else if ( *_pC == '\\') 
	    { 
		_stringBuf[i] = '\\'; 
	    } 
	    else 
	    { 
		Chain msg = Chain("Invalid escape character <") + Chain(*_pC) + Chain(">"); 
		throw Exception(EXLOC, msg);
	    } 
	} 
	else
	{ 
	    _stringBuf[i] = *_pC; 
	} 
	
	i++; 
	
	if (i == POSTGRES_MAXSTRINGLEN) 
	{ 
	    Chain msg("Stringbuf exceeded"); 
	    throw Exception(EXLOC, msg);
	}
	
	_pC++;
	
	if ( *_pC == 0 )
	{
	    Chain msg("Unterminated string"); 
	    throw Exception(EXLOC, msg);
	}
    }
    
    _pC++; 

    _stringBufLen = i+1;
    _stringBuf[i] = 0; 

    return; 
}

void PostgresAction::backChar() 
{
    if ( _pC != 0 )
	_pC--;
}
    
void PostgresAction::printTokenList() 
{    
    cout << "TokenList is " << endl;
	
    ListT<Chain> tlist =  getTokenList();
    Chain* pS = tlist.First();
    while (pS)
    {
	cout << *pS << endl;
	pS = tlist.Next();
    }    
}

void PostgresAction::cleanUp() 
{

}

void PostgresAction::postAction()
{
    cout << "Post processing " << endl;
    Chain *pStmt = _postList.First();
    while ( pStmt )
    {
	if ( _pCegoNet )
	{
	    cout << ".";
	    cout.flush();
	    _pCegoNet->doQuery(*pStmt);
	}
	else
	    cout << *pStmt << endl;
	pStmt = _postList.Next();
    }

    cout << endl;
} 

void PostgresAction::execCreate() 
{

    Chain stmt = Chain("create table ") + _table + Chain(" (");

    Chain *pCol = _colList.First();
    while ( pCol )
    {
	stmt += Chain(" ") + *pCol;
	pCol = _colList.Next();
	if ( pCol )
	    stmt += Chain(",");
	
    }
    stmt += Chain(");");
    _colList.Empty();   



    // cout << "Create = " << stmt << endl;
    if ( _pCegoNet )
    {
	cout << "Creating table " << _table << endl;
	_pCegoNet->doQuery(stmt);
    }
    else
	cout << stmt << endl;
    
}

void PostgresAction::insertPreAction()
{
    if ( _pCegoNet )
    {
	cout << "Inserting into " << _table << " ";
	cout.flush();
    }
} 

void PostgresAction::insertPostAction()
{
    if ( _pCegoNet )
    {
	cout << endl;
    }
} 

void PostgresAction::execInsert() 
{

    Chain stmt = Chain("insert into ") + _table + Chain(" values (");

    Chain *pVal = _valList.First();
    int pos=0;
    while ( pVal )
    {
	TypeInfo* pTI = getTypeInfo(_table, pos);
	if ( pTI == 0 )
	    throw Exception(EXLOC, "No type info");

	if ( pTI->getType() == Chain("datetime") && *pVal != Chain("null") )
	{
	    if ( pVal->subChain(2,5) == Chain("1899") )
		stmt += Chain(" null");
	    else
		stmt += Chain(" date('%Y-%m-%d %H:%M:%S',") + *pVal + Chain(")");
	}
	else
	    if ( pVal->length() > MAXCGSTRLEN )
		stmt += Chain(" 'XXX'"); //  + pVal->subChain(0, MAXCGSTRLEN);
	    else
		stmt += Chain(" ") + *pVal;

	      
	pVal = _valList.Next();
	if ( pVal )
	    stmt += Chain(",");	
	pos++;
    }
    stmt += Chain(");");
    _valList.Empty();   


    if ( _pCegoNet )
    {
	cout << ".";
	cout.flush();
	_pCegoNet->doQuery(stmt);
    }
    else
	cout << stmt << endl;

}

void PostgresAction::storeTable() 
{
    Chain* pS = getTokenList().First();
    _table = pS->cutTrailing("`");
}

void PostgresAction::execDrop()
{
    Chain stmt = Chain("drop if exists table ") + _table +  Chain(";");

    if ( _pCegoNet )
	_pCegoNet->doQuery(stmt); 
    else
	cout << stmt << endl;

}

void PostgresAction::dropOpt1()
{    
}

void PostgresAction::dropOpt2()
{
}

void PostgresAction::storeAttrColumn()
{
    Chain* pS = getTokenList().First();
    
    Chain attrName =  pS->cutTrailing("`");

    if ( attrName == Chain("sync") )
	attrName = Chain("a_sync");
    else if ( attrName == Chain("start") )
	attrName = Chain("a_start");

    Chain colString = attrName + Chain(" ") + _dataTypeString;
    if ( _nullOptString.length() > 0 )
	colString += Chain(" ") + _nullOptString;

    if ( _defaultOptString.length() > 0 )
	colString += Chain(" ") + _defaultOptString;
 
    // cout << "ColString = " << colString << endl;

    putTypeInfo( TypeInfo(_table, _colList.Size(), _dataTypeString) );
    _colList.Insert(colString);
}

void PostgresAction::storePrimaryKey()
{
    Chain stmt = Chain("create primary index on ") + _table + Chain("(");

    Chain *pKeyCol = _keyColList.First();
    while ( pKeyCol )
    {
	stmt += Chain(" ") + *pKeyCol;
	pKeyCol = _keyColList.Next();
	if ( pKeyCol )
	    stmt += Chain(",");	
    }
    stmt += Chain(");");

    _keyColList.Empty();

    _postList.Insert(stmt);
}

void PostgresAction::storeUniqueKey()
{

    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    pS = getTokenList().Next();

    Chain btreeName;
    btreeName = pS->cutTrailing("`");
    
    Chain stmt = Chain("create unique btree ") + btreeName + Chain(" on ") + _table + Chain("(");

    Chain *pKeyCol = _keyColList.First();
    while ( pKeyCol )
    {
	stmt += Chain(" ") + *pKeyCol;
	pKeyCol = _keyColList.Next();
	if ( pKeyCol )
	    stmt += Chain(",");	
    }
    stmt += Chain(");");

    _keyColList.Empty();

    _postList.Insert(stmt);
}

void PostgresAction::storeKey()
{
    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    pS = getTokenList().Next();

    Chain btreeName = pS->cutTrailing("`");

    Chain stmt = Chain("create btree ") + btreeName + Chain(" on ") + _table + Chain("(");

    Chain *pKeyCol = _keyColList.First();
    while ( pKeyCol )
    {
	stmt += Chain(" ") + *pKeyCol;
	pKeyCol = _keyColList.Next();
	if ( pKeyCol )
	    stmt += Chain(",");	
    }
    stmt += Chain(");");

    _keyColList.Empty();

    _postList.Insert(stmt);
}



void PostgresAction::storeKeyColumn()
{
    Chain* pS = getTokenList().First();
    _keyColList.Insert(pS->cutTrailing("`"));
}

void PostgresAction::storeNullOpt()
{
    _nullOptString = "";
}

void PostgresAction::storeNotNullOpt()
{
    _nullOptString = "not null";
}

void PostgresAction::storeDefaultOpt()
{
}

void PostgresAction::storeDefaultNullOpt()
{
}

void PostgresAction::insertConstValue()
{
    _valList.Insert(_constValue);
}

void PostgresAction::storeIntType()
{
    if ( _dim < 10 )
	_dataTypeString = Chain("int");
    else
	_dataTypeString = Chain("long");
}

void PostgresAction::storeDecimalType()
{
    _dataTypeString = Chain("fixed(") + Chain(_decDim) + Chain(")");
}

void PostgresAction::storeUnsignedType()
{
}

void PostgresAction::storeVarcharType()
{

    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    
    _dataTypeString = Chain("string(") + Chain(*pS) + Chain(")");
}

void PostgresAction::storeTimestampType()
{
    _dataTypeString = Chain("datetime");
}

void PostgresAction::storeDatetimeType()
{
    _dataTypeString = Chain("datetime");
}

void PostgresAction::storeMediumTextType()
{
    _dataTypeString = Chain("string(") + Chain(MAXCGSTRLEN) + Chain(")");
}



void PostgresAction::storeEnumType()
{
    // for now, we map enum to string 100
    _dataTypeString = Chain("string(100)");
}


void PostgresAction::storeIntValue()
{
    Chain* pS = getTokenList().First();
    
    if ( pS->length() < 10 )
	_constValue = *pS;
    else
	_constValue = Chain("(long)") + *pS;
}

void PostgresAction::storeNegIntValue()
{
    Chain* pS = getTokenList().First();
    
    if ( pS->length() < 10 )
	_constValue = Chain("-") + *pS;
    else
	_constValue = Chain("(long)-") + *pS;
}

void PostgresAction::storeVarcharValue()
{
    _constValue = Chain("'") + Chain(_stringBuf) + Chain("'");
}

void PostgresAction::storeNullValue()
{
    _constValue = Chain("null");
}

void PostgresAction::storeDim()
{
    
    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    _dim = pS->asInteger();
}

void PostgresAction::storeDecimalDim()
{
    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    _decDim = pS->asInteger();
    pS = getTokenList().Next();
    pS = getTokenList().Next();
    _dim = pS->asInteger();
}

void PostgresAction::putTypeInfo(TypeInfo ti)
{
    // cout << "Putting type info " << ti.getTable() << " " << ti.getPos() << " " << ti.getType() << endl;
    _typeList.Insert(ti);
}

PostgresAction::TypeInfo* PostgresAction::getTypeInfo(const Chain& tableName, int pos)
{
    TypeInfo* pTI = _typeList.Find( TypeInfo(tableName, pos) );
    return pTI;
}
